@@ -19,12 +19,20 @@
   private: \
     static O sm_instance
 
+//----------------------------------------------------------------------------
 #define STRATEGY_OBJECT(O) STATE_OBJECT(O)
 
 //----------------------------------------------------------------------------
 #define SINGLETON_OBJECT(O) STATE_OBJECT(O)
 
 
+//----------------------------------------------------------------------------
+#define DECLARE_PIMPL \
+  private: \
+    class PImpl; \
+    PImpl *m_pImpl
+
+
 //----------------------------------------------------------------------------
 #if defined __GNUC__ && ! defined __LINT__
 #define MIN(X, Y) ((X) <? (Y))
