@@ -81,7 +81,7 @@ void SDLFrameRate::delay()
 //============================================================================
 
 //----------------------------------------------------------------------------
-void SDL_CALLS::Init(Uint32 flags) throw (SDLException)
+void SDL_CALLS::Init(Uint32 flags)
 {
     int rc = SDL_Init(flags);
     if (rc != 0)
@@ -93,7 +93,6 @@ void SDL_CALLS::Init(Uint32 flags) throw (SDLException)
 //----------------------------------------------------------------------------
 void SDL_CALLS::BlitSurface(const SDL_Surface *src, const SDL_Rect *srcrect,
                             SDL_Surface *dst, const SDL_Rect *dstrect)
-    throw (SDLException)
 {
     SDL_Rect dest;
     if (dstrect)
@@ -119,7 +118,6 @@ void SDL_CALLS::BlitSurface(const SDL_Surface *src, const SDL_Rect *srcrect,
 SDL_Surface *SDL_CALLS::ConvertSurface(const SDL_Surface *src,
                                        const SDL_PixelFormat *fmt,
                                        Uint32 flags)
-    throw (SDLException)
 {
     SDL_Surface *s = SDL_ConvertSurface(
         const_cast<SDL_Surface*>(src),
@@ -136,7 +134,6 @@ SDL_Surface *SDL_CALLS::ConvertSurface(const SDL_Surface *src,
 SDL_Surface *SDL_CALLS::CreateRGBSurface(
     Uint32 flags, int width, int height, int depth,
     Uint32 Rmask, Uint32 Gmask, Uint32 Bmask, Uint32 Amask)
-    throw (SDLException)
 {
     SDL_Surface *s = SDL_CreateRGBSurface(flags, width, height, depth,
                                           Rmask, Gmask, Bmask, Amask);
@@ -147,6 +144,22 @@ SDL_Surface *SDL_CALLS::CreateRGBSurface(
     return s;
 }
 
+//----------------------------------------------------------------------------
+SDL_Surface *SDL_CALLS::CreateRGBSurfaceFrom(
+    void *pixels, int width, int height, int depth, int pitch,
+    Uint32 Rmask, Uint32 Gmask, Uint32 Bmask, Uint32 Amask)
+{
+    SDL_Surface *s = SDL_CreateRGBSurfaceFrom(
+        pixels, width, height, depth, pitch,
+        Rmask, Gmask, Bmask, Amask);
+    if (s == NULL)
+    {
+        throw SDLException("SDL_CreateRGBSurfaceFrom()");
+    }
+    return s;
+}
+
+
 //----------------------------------------------------------------------------
 void SDL_CALLS::UpdateRect(Sint32 x, Sint32 y, Sint32 w, Sint32 h)
 {
@@ -161,7 +174,7 @@ void SDL_CALLS::UpdateRects(int numrects, const SDL_Rect *rects)
 }
 
 //----------------------------------------------------------------------------
-void SDL_CALLS::Flip() throw (SDLException)
+void SDL_CALLS::Flip()
 {
     int rc = SDL_Flip(SDL_GetVideoSurface());
     if (rc != 0)
@@ -177,7 +190,7 @@ void SDL_CALLS::FreeSurface(SDL_Surface *surface)
 }
 
 //----------------------------------------------------------------------------
-SDL_Surface *SDL_CALLS::LoadBMP(const char *file) throw (SDLException)
+SDL_Surface *SDL_CALLS::LoadBMP(const char *file)
 {
     SDL_Surface *s = SDL_LoadBMP(file);
     if (s == NULL)
@@ -189,7 +202,6 @@ SDL_Surface *SDL_CALLS::LoadBMP(const char *file) throw (SDLException)
 
 //----------------------------------------------------------------------------
 void SDL_CALLS::SetAlpha(SDL_Surface *surface, Uint32 flag, Uint8 alpha)
-    throw (SDLException)
 {
     int rc = SDL_SetAlpha(surface, flag, alpha);
     if (rc != 0)
@@ -200,7 +212,6 @@ void SDL_CALLS::SetAlpha(SDL_Surface *surface, Uint32 flag, Uint8 alpha)
 
 //----------------------------------------------------------------------------
 void SDL_CALLS::SetColorKey(SDL_Surface *surface, Uint32 flag, Uint32 key)
-    throw (SDLException)
 {
     int rc = SDL_SetColorKey(surface, flag, key);
     if (rc != 0)
@@ -212,7 +223,6 @@ void SDL_CALLS::SetColorKey(SDL_Surface *surface, Uint32 flag, Uint32 key)
 //----------------------------------------------------------------------------
 void SDL_CALLS::SetPalette(SDL_Surface *surface, int flags,
                            SDL_Color *colors, int firstcolor, int ncolors)
-    throw (SDLException)
 {
     int rc = SDL_SetPalette(surface, flags, colors, firstcolor, ncolors);
     if (rc != 1)
@@ -224,7 +234,6 @@ void SDL_CALLS::SetPalette(SDL_Surface *surface, int flags,
 //----------------------------------------------------------------------------
 SDL_Surface *SDL_CALLS::SetVideoMode(int width, int height,
                                      int bpp, Uint32 flags)
-    throw (SDLException)
 {
     SDL_Surface *s = SDL_SetVideoMode(width, height, bpp, flags);
     if (s == NULL)
@@ -239,7 +248,6 @@ SDL_Surface *SDL_CALLS::SetVideoMode(int width, int height,
 void SDL_CALLS::FillRect(SDL_Surface *surface,
                          const SDL_Rect *rect,
                          Uint32 color)
-    throw (SDLException)
 {
     SDL_Rect dest;
     if (rect)
@@ -262,7 +270,6 @@ void SDL_CALLS::FillRect(SDL_Surface *surface,
 //----------------------------------------------------------------------------
 SDL_Surface *SDL_CALLS::rotozoomSurface(const SDL_Surface *src, double angle,
                                         double zoom, int smooth)
-    throw (SDLException)
 {
     SDL_Surface *s = ::rotozoomSurface(
         const_cast<SDL_Surface*>(src), angle, zoom, smooth);
@@ -275,7 +282,6 @@ SDL_Surface *SDL_CALLS::rotozoomSurface(const SDL_Surface *src, double angle,
 
 //----------------------------------------------------------------------------
 void SDL_CALLS::PushEvent(const SDL_Event &event)
-    throw (SDLException)
 {
     int rc = SDL_PushEvent(&const_cast<SDL_Event&>(event));
     if (rc != 0)
@@ -293,7 +299,6 @@ void SDL_CALLS::PushEvent(const SDL_Event &event)
 //----------------------------------------------------------------------------
 void SDL_MIXER_CALLS::OpenAudio(int rate, Uint16 format,
                                 int channels, int buffers)
-    throw (SDLException)
 {
     int rc = Mix_OpenAudio(rate, format, channels, buffers);
     if (rc != 0)
@@ -304,7 +309,6 @@ void SDL_MIXER_CALLS::OpenAudio(int rate, Uint16 format,
 
 //----------------------------------------------------------------------------
 void SDL_MIXER_CALLS::QuerySpec(int* rate, Uint16* format, int* channels)
-    throw (SDLException)
 {
     int rc = Mix_QuerySpec(rate, format, channels);
     if (rc == 0)
@@ -326,7 +330,7 @@ int SDL_MIXER_CALLS::AllocateChannels(int num)
 }
 
 //----------------------------------------------------------------------------
-Mix_Chunk* SDL_MIXER_CALLS::LoadWAV(const char* file) throw (SDLException)
+Mix_Chunk* SDL_MIXER_CALLS::LoadWAV(const char* file)
 {
     Mix_Chunk *chunk = Mix_LoadWAV(file);
     if (chunk == NULL)
@@ -339,7 +343,6 @@ Mix_Chunk* SDL_MIXER_CALLS::LoadWAV(const char* file) throw (SDLException)
 
 //----------------------------------------------------------------------------
 int SDL_MIXER_CALLS::PlayChannel(const Mix_Chunk* chunk, int loops)
-    throw (SDLException)
 {
     int rc = Mix_PlayChannel(-1, const_cast<Mix_Chunk*>(chunk), loops);
     if (rc == -1)
@@ -351,13 +354,13 @@ int SDL_MIXER_CALLS::PlayChannel(const Mix_Chunk* chunk, int loops)
 }
 
 //----------------------------------------------------------------------------
-void SDL_MIXER_CALLS::HaltChannel(int channel) throw (SDLException)
+void SDL_MIXER_CALLS::HaltChannel(int channel)
 {
     (void)Mix_HaltChannel(channel);  // Always returns 0.
 }
 
 //----------------------------------------------------------------------------
-Mix_Music* SDL_MIXER_CALLS::LoadMUS(const char* track) throw (SDLException)
+Mix_Music* SDL_MIXER_CALLS::LoadMUS(const char* track)
 {
     Mix_Music *m = Mix_LoadMUS(track);
     if (m == NULL)
@@ -370,7 +373,6 @@ Mix_Music* SDL_MIXER_CALLS::LoadMUS(const char* track) throw (SDLException)
 
 //----------------------------------------------------------------------------
 void SDL_MIXER_CALLS::PlayMusic(const Mix_Music* music, int loops)
-    throw (SDLException)
 {
     int rc = Mix_PlayMusic(const_cast<Mix_Music*>(music), loops);
     if (rc == -1)
@@ -412,6 +414,9 @@ void SDL_MIXER_CALLS::FreeMusic(Mix_Music* music)
 //----------------------------------------------------------------------------
 Uint32 SDL_TOOLS::getPixel(const SDL_Surface *surface, Uint16 x, Uint16 y)
 {
+    assert(x < surface->w);
+    assert(y < surface->h);
+
     int bpp = surface->format->BytesPerPixel;
     assert(bpp == 1 || bpp == 2 || bpp == 4);
 
@@ -498,7 +503,6 @@ void SDL_TOOLS::unite(const SDL_Rect &r1, const SDL_Rect &r2, SDL_Rect &u)
 
 //----------------------------------------------------------------------------
 static SDL_Surface *rotate8bpp_(const SDL_Surface *src, Uint16 angle)
-    throw (SDLException)
 {
     std::auto_ptr<SDL_Surface> dst;
 
@@ -586,7 +590,6 @@ static SDL_Surface *rotate8bpp_(const SDL_Surface *src, Uint16 angle)
 
 //----------------------------------------------------------------------------
 SDL_Surface *SDL_TOOLS::rotate(const SDL_Surface *src, Uint16 angle)
-    throw (SDLException)
 {
     switch (src->format->BitsPerPixel)
     {
@@ -655,7 +658,6 @@ void SDL_TOOLS::getCentre(const SDL_Rect &r, Sint16 &x, Sint16 &y)
 //----------------------------------------------------------------------------
 SDL_Surface *SDL_TOOLS::loadBMPWithColorKey(const char *file,
                                             Uint8 r, Uint8 g, Uint8 b)
-    throw (SDLException)
 {
     SDL_Surface *s = SDL_CALLS::LoadBMP(file);
     SDL_CALLS::SetColorKey(
@@ -663,3 +665,12 @@ SDL_Surface *SDL_TOOLS::loadBMPWithColorKey(const char *file,
     return s;
 }
 
+//----------------------------------------------------------------------------
+SDL_Surface *SDL_TOOLS::cloneSurface(const SDL_Surface *s)
+{
+    SDL_Surface *clone = SDL_CALLS::CreateRGBSurface(
+        SDL_SWSURFACE, s->w, s->h, 32);
+    SDL_CALLS::BlitSurface(s, NULL, clone, NULL);
+
+    return clone;
+}
