@@ -16,6 +16,13 @@
 // Implementation of SyscallException.h
 //============================================================================
 
+//----------------------------------------------------------------------------
+SyscallException::SyscallException(const char *message)
+        : Exception(message)
+{
+    m_errno = errno;
+}
+
 //----------------------------------------------------------------------------
 SyscallException::SyscallException(const std::string &message)
         : Exception(message)
@@ -23,6 +30,13 @@ SyscallException::SyscallException(const std::string &message)
     m_errno = errno;
 }
 
+//----------------------------------------------------------------------------
+SyscallException::SyscallException(const char *message, int error)
+        : Exception(message)
+{
+    m_errno = error;
+}
+
 //----------------------------------------------------------------------------
 SyscallException::SyscallException(const std::string &message, int error)
         : Exception(message)
@@ -49,7 +63,6 @@ std::string SyscallException::toString() const
 
 //----------------------------------------------------------------------------
 void SYSTEM_CALL::stat(const char *file, struct stat *buf)
-    throw (SyscallException)
 {
     if (::stat(file, buf) != 0)
     {
@@ -60,7 +73,6 @@ void SYSTEM_CALL::stat(const char *file, struct stat *buf)
 
 //----------------------------------------------------------------------------
 void SYSTEM_CALL::lstat(const char *file, struct stat *buf)
-    throw (SyscallException)
 {
     if (::lstat(file, buf) != 0)
     {
@@ -76,32 +88,26 @@ void SYSTEM_CALL::lstat(const char *file, struct stat *buf)
 //============================================================================
 
 //----------------------------------------------------------------------------
-class FilePImpl
+class File::PImpl
 {
 public:
     //------------------------------------------------------------------------
-    FilePImpl(const char *name, const char *mode)
-        throw (SyscallException);
-    ~FilePImpl();
+    PImpl(const char *name, const char *mode);
+    ~PImpl();
 
     //------------------------------------------------------------------------
     const char *getName() const;
 
     bool isEOF() const;
 
-    void seek(long offset)
-        throw (SyscallException);
+    void seek(long offset);
 
-    size_t read(void *buffer, size_t len)
-        throw (SyscallException);
-    size_t write(const void *buffer, size_t len)
-        throw (SyscallException);
+    size_t read(void *buffer, size_t len);
+    size_t write(const void *buffer, size_t len);
 
-    int puts(const char *s)
-        throw (SyscallException);
+    int puts(const char *s);
 
-    void readline(char *buffer, size_t len)
-        throw (SyscallException);
+    void readline(char *buffer, size_t len);
 
 private:
     //------------------------------------------------------------------------
@@ -111,8 +117,7 @@ private:
 
 
 //----------------------------------------------------------------------------
-FilePImpl::FilePImpl(const char *name, const char *mode)
-    throw (SyscallException)
+File::PImpl::PImpl(const char *name, const char *mode)
 {
     m_name = name;
     m_file = fopen(name, mode);
@@ -125,27 +130,26 @@ FilePImpl::FilePImpl(const char *name, const char *mode)
 }
 
 //----------------------------------------------------------------------------
-FilePImpl::~FilePImpl()
+File::PImpl::~PImpl()
 {
     fclose(m_file);
     m_file = NULL;
 }
 
 //----------------------------------------------------------------------------
-const char *FilePImpl::getName() const
+const char *File::PImpl::getName() const
 {
     return m_name;
 }
 
 //----------------------------------------------------------------------------
-bool FilePImpl::isEOF() const
+bool File::PImpl::isEOF() const
 {
     return feof(m_file);
 }
 
 //----------------------------------------------------------------------------
-void FilePImpl::seek(long offset)
-    throw (SyscallException)
+void File::PImpl::seek(long offset)
 {
     int rc = fseek(m_file, offset, SEEK_SET);
     if (rc == -1)
@@ -155,7 +159,7 @@ void FilePImpl::seek(long offset)
 }
 
 //----------------------------------------------------------------------------
-size_t FilePImpl::read(void *buffer, size_t len) throw (SyscallException)
+size_t File::PImpl::read(void *buffer, size_t len)
 {
     size_t rc = fread(buffer, 1, len, m_file);
     if ((rc != len) && !isEOF())
@@ -171,8 +175,7 @@ size_t FilePImpl::read(void *buffer, size_t len) throw (SyscallException)
 }
 
 //----------------------------------------------------------------------------
-size_t FilePImpl::write(const void *buffer, size_t len)
-    throw (SyscallException)
+size_t File::PImpl::write(const void *buffer, size_t len)
 {
     size_t rc = fwrite(buffer, 1, len, m_file);
     if (rc != len)
@@ -188,8 +191,7 @@ size_t FilePImpl::write(const void *buffer, size_t len)
 }
 
 //----------------------------------------------------------------------------
-int FilePImpl::puts(const char *s)
-    throw (SyscallException)
+int File::PImpl::puts(const char *s)
 {
     int rc = fputs(s, m_file);
     if (rc == EOF)
@@ -205,7 +207,7 @@ int FilePImpl::puts(const char *s)
 }
 
 //----------------------------------------------------------------------------
-void FilePImpl::readline(char *buffer, size_t len) throw (SyscallException)
+void File::PImpl::readline(char *buffer, size_t len)
 {
     fgets(buffer, len, m_file);
     char *newline = strchr(buffer, '\n');
@@ -217,9 +219,9 @@ void FilePImpl::readline(char *buffer, size_t len) throw (SyscallException)
 
 
 //----------------------------------------------------------------------------
-File::File(const char *name, const char *mode) throw (SyscallException)
+File::File(const char *name, const char *mode)
 {
-    m_pImpl = new FilePImpl(name, mode);
+    m_pImpl = new PImpl(name, mode);
 }
 
 //----------------------------------------------------------------------------
@@ -243,14 +245,13 @@ bool File::isEOF() const
 
 //----------------------------------------------------------------------------
 void File::seek(long offset)
-    throw (SyscallException)
 {
     m_pImpl->seek(offset);
 }
 
 
 //----------------------------------------------------------------------------
-uint8_t File::readUint8() throw (SyscallException)
+uint8_t File::readUint8()
 {
     uint8_t value;
     read(&value, 1);
@@ -258,24 +259,24 @@ uint8_t File::readUint8() throw (SyscallException)
 }
 
 //----------------------------------------------------------------------------
-void File::writeUint8(uint8_t value) throw (SyscallException)
+void File::writeUint8(uint8_t value)
 {
     write(&value, 1);
 }
 
 //----------------------------------------------------------------------------
-uint16_t File::readUint16() throw (SyscallException)
+uint16_t File::readUint16()
 {
-    char buffer[2];
+    unsigned char buffer[2];
     read(buffer, 2);
     uint16_t value = (uint16_t)buffer[0] << 8 | (uint16_t)buffer[1];
     return value;
 }
 
 //----------------------------------------------------------------------------
-void File::writeUint16(uint16_t value) throw (SyscallException)
+void File::writeUint16(uint16_t value)
 {
-    char buffer[2];
+    unsigned char buffer[2];
     buffer[0] = (value & 0xff00) >> 8;
     buffer[1] = value & 0x00ff;
     write(buffer, 2);
@@ -283,38 +284,38 @@ void File::writeUint16(uint16_t value) throw (SyscallException)
 
 
 //----------------------------------------------------------------------------
-size_t File::read(void *buffer, size_t len) throw (SyscallException)
+size_t File::read(void *buffer, size_t len)
 {
     return m_pImpl->read(buffer, len);
 }
 
 //----------------------------------------------------------------------------
-size_t File::write(const void *buffer, size_t len) throw (SyscallException)
+size_t File::write(const void *buffer, size_t len)
 {
     return m_pImpl->write(buffer, len);
 }
 
 //----------------------------------------------------------------------------
-int File::puts(const char *s) throw (SyscallException)
+int File::puts(const char *s)
 {
     return m_pImpl->puts(s);
 }
 
 //----------------------------------------------------------------------------
-int File::puts(const std::string &s) throw (SyscallException)
+int File::puts(const std::string &s)
 {
     return m_pImpl->puts(s.c_str());
 }
 
 //----------------------------------------------------------------------------
-void File::readline(char *buffer, size_t len) throw (SyscallException)
+void File::readline(char *buffer, size_t len)
 {
     m_pImpl->readline(buffer, len);
 }
 
 
 //----------------------------------------------------------------------------
-void File::unlink(const char *name) throw (SyscallException)
+void File::unlink(const char *name)
 {
     if ((::unlink(name) != 0) && (errno != ENOENT))
     {
@@ -325,7 +326,7 @@ void File::unlink(const char *name) throw (SyscallException)
 }
 
 //----------------------------------------------------------------------------
-void File::copy(const char *src, const char *dst) throw (SyscallException)
+void File::copy(const char *src, const char *dst)
 {
     File srcFile(src, "r");
     File dstFile(dst, "w");
@@ -351,15 +352,14 @@ void File::copy(const char *src, const char *dst) throw (SyscallException)
 //============================================================================
 
 //----------------------------------------------------------------------------
-class DirectoryPImpl
+class Directory::PImpl
 {
 public:
     //------------------------------------------------------------------------
     class Handle
     {
     public:
-        Handle(const char *name)
-            throw (SyscallException);
+        Handle(const char *name);
         ~Handle();
 
         void rewind();
@@ -375,8 +375,8 @@ public:
     typedef Entries::const_iterator const_iterator;
     
     //------------------------------------------------------------------------
-    DirectoryPImpl();
-    ~DirectoryPImpl();
+    PImpl();
+    ~PImpl();
 
     //------------------------------------------------------------------------
     inline void clear() { m_entries.clear(); }
@@ -390,8 +390,7 @@ public:
     inline const_iterator end() const { return m_entries.end(); }
 
     //------------------------------------------------------------------------
-    void read(const char *name)
-        throw (SyscallException);
+    void read(const char *name);
 
     const char *getName() const;
 
@@ -409,17 +408,17 @@ private:
 
 
 //----------------------------------------------------------------------------
-DirectoryPImpl::DirectoryPImpl()
+Directory::PImpl::PImpl()
 {
 }
 
 //----------------------------------------------------------------------------
-DirectoryPImpl::~DirectoryPImpl()
+Directory::PImpl::~PImpl()
 {
 }
 
 //----------------------------------------------------------------------------
-void DirectoryPImpl::read(const char *name) throw (SyscallException)
+void Directory::PImpl::read(const char *name)
 {
     clear();
 
@@ -440,13 +439,13 @@ void DirectoryPImpl::read(const char *name) throw (SyscallException)
 }
 
 //----------------------------------------------------------------------------
-const char *DirectoryPImpl::getName() const
+const char *Directory::PImpl::getName() const
 {
     return m_name;
 }
 
 //----------------------------------------------------------------------------
-bool DirectoryPImpl::hasDirectory(const char *name) const
+bool Directory::PImpl::hasDirectory(const char *name) const
 {
     try
     {
@@ -472,7 +471,7 @@ bool DirectoryPImpl::hasDirectory(const char *name) const
 }
 
 //----------------------------------------------------------------------------
-bool DirectoryPImpl::hasFile(const char *name) const
+bool Directory::PImpl::hasFile(const char *name) const
 {
     try
     {
@@ -498,8 +497,8 @@ bool DirectoryPImpl::hasFile(const char *name) const
 }
 
 //----------------------------------------------------------------------------
-void DirectoryPImpl::getFilesWithSuffix(const char* suffix,
-                                        Directory::FileList &list) const
+void Directory::PImpl::getFilesWithSuffix(const char* suffix,
+                                          Directory::FileList &list) const
 {
     list.clear();
     for (const_iterator iter = begin(); iter != end(); ++iter)
@@ -514,8 +513,7 @@ void DirectoryPImpl::getFilesWithSuffix(const char* suffix,
 }
 
 //----------------------------------------------------------------------------
-DirectoryPImpl::Handle::Handle(const char *name)
-    throw (SyscallException)
+Directory::PImpl::Handle::Handle(const char *name)
 {
     m_dir = opendir(name);
     if (m_dir == NULL)
@@ -526,7 +524,7 @@ DirectoryPImpl::Handle::Handle(const char *name)
 }
 
 //----------------------------------------------------------------------------
-DirectoryPImpl::Handle::~Handle()
+Directory::PImpl::Handle::~Handle()
 {
     if (m_dir)
     {
@@ -536,13 +534,13 @@ DirectoryPImpl::Handle::~Handle()
 }
 
 //----------------------------------------------------------------------------
-void DirectoryPImpl::Handle::rewind()
+void Directory::PImpl::Handle::rewind()
 {
     rewinddir(m_dir);
 }
 
 //----------------------------------------------------------------------------
-struct dirent *DirectoryPImpl::Handle::read()
+struct dirent *Directory::PImpl::Handle::read()
 {
     return readdir(m_dir);
 }
@@ -552,14 +550,13 @@ struct dirent *DirectoryPImpl::Handle::read()
 //----------------------------------------------------------------------------
 Directory::Directory()
 {
-    m_pImpl = new DirectoryPImpl();
+    m_pImpl = new PImpl();
 }
 
 //----------------------------------------------------------------------------
 Directory::Directory(const char *name)
-    throw (SyscallException)
 {
-    m_pImpl = new DirectoryPImpl();
+    m_pImpl = new PImpl();
     read(name);
 }
 
@@ -571,7 +568,6 @@ Directory::~Directory()
 
 //----------------------------------------------------------------------------
 void Directory::read(const char *name)
-    throw (SyscallException)
 {
     m_pImpl->read(name);
 }
@@ -597,7 +593,6 @@ bool Directory::hasFile(const char *name) const
 
 //----------------------------------------------------------------------------
 bool Directory::hasDirectoryInCwd(const char *name)
-    throw (SyscallException)
 {
     Directory d(".");
     return d.hasDirectory(name);
@@ -605,7 +600,6 @@ bool Directory::hasDirectoryInCwd(const char *name)
 
 //----------------------------------------------------------------------------
 bool Directory::hasFileInCwd(const char *name)
-    throw (SyscallException)
 {
     Directory d(".");
     return d.hasFile(name);
@@ -619,7 +613,6 @@ void Directory::getFilesWithSuffix(const char *suffix, FileList &list) const
 
 //----------------------------------------------------------------------------
 void Directory::chdir(const char *name)
-    throw (SyscallException)
 {
     if (::chdir(name) != 0)
     {
@@ -631,7 +624,6 @@ void Directory::chdir(const char *name)
 
 //----------------------------------------------------------------------------
 void Directory::mkdir(const char *name, mode_t mode, bool recursive)
-    throw (SyscallException)
 {
     while (::mkdir(name, mode) != 0)
     {
@@ -656,7 +648,6 @@ void Directory::mkdir(const char *name, mode_t mode, bool recursive)
 
 //----------------------------------------------------------------------------
 void Directory::rmdir(const char *name, bool recursive)
-    throw (SyscallException)
 {
     Directory d;
     while (::rmdir(name) != 0)
@@ -665,7 +656,7 @@ void Directory::rmdir(const char *name, bool recursive)
         {
             d.read(name);
 
-            for (DirectoryPImpl::const_iterator iter = d.m_pImpl->begin();
+            for (PImpl::const_iterator iter = d.m_pImpl->begin();
                  iter != d.m_pImpl->end(); ++iter)
             {
                 std::string newName(name);
@@ -696,13 +687,12 @@ void Directory::rmdir(const char *name, bool recursive)
 
 //----------------------------------------------------------------------------
 void Directory::copy(const char *src, const char *dst)
-    throw (SyscallException)
 {
     mkdir(dst);
 
     Directory srcDir(src);
 
-    for (DirectoryPImpl::const_iterator iter = srcDir.m_pImpl->begin();
+    for (PImpl::const_iterator iter = srcDir.m_pImpl->begin();
          iter != srcDir.m_pImpl->end(); ++iter)
     {
         std::string srcName(src);
