@@ -1,23 +1,13 @@
 #include <cmath>
+#include <sstream>
+#include <iomanip>
 
+#include "XMLException.h"
 #include "Configuration.h"
 #include "Tiles.h"
-#include "Barriers.h"
-#include "BlackHole.h"
-#include "Magnets.h"
-#include "ParticleFountains.h"
-#include "Platform.h"
-#include "SAMBattery.h"
-#include "Turrets.h"
-#include "Mortars.h"
-#include "Ship.h"
-#include "Missile.h"
-#include "Crates.h"
-#include "Projectile.h"
-#include "Grenade.h"
-#include "Particle.h"
-#include "Grinder.h"
-#include "Tank.h"
+
+#include "ObjectRepository.h"
+#include "AllObjects.h"
 #include "GameControlBase.h"
 
 #include "PlayGroundVisitors.h"
@@ -110,15 +100,13 @@ void CreateGravityVisitor::do_visit(const XMLNode *n)
 //----------------------------------------------------------------------------
 void CreateDecorationsVisitor::do_visit(const XMLNode *n)
 {
-    PlayGround *playGround = PlayGround::getInstance();
-
     if (n->getName() == "decorations")
     {
         n->acceptAllChilds(*this);
     }
     else
     {
-        playGround->addObject(DecorationBase::create(n));
+        ObjectRepository::getInstance()->addObject(DecorationBase::create(n));
     }
 }
 
@@ -139,25 +127,10 @@ PlayGround::~PlayGround()
     ZAP_SURFACE(m_collisionSurface);
     ZAP_SURFACE(m_mapSurface);
     ZAP_SURFACE(m_shadowSurface);
-
-    for (ObjectIter iter = m_objects.begin();
-         iter != m_objects.end(); ++iter)
-    {
-        delete iter->object;
-    }
-    m_objects.clear();
-
-    for (ParticleIter iter = m_particles.begin();
-         iter != m_particles.end(); ++iter)
-    {
-        delete *iter;
-    }
-    m_particles.clear();
 }
 
 //----------------------------------------------------------------------------
 void PlayGround::init(const char *mission, const XMLNode *playGroundNode)
-    throw (Exception)
 {
     destroy();
 
@@ -183,60 +156,16 @@ void PlayGround::destroy()
 }
 
 
-//----------------------------------------------------------------------------
-void PlayGround::addObject(ObjectBase *o)
-{
-    ObjectEntry e = { o, false, true };
-    m_objects.push_back(e);
-}
-
-//----------------------------------------------------------------------------
-void PlayGround::addParticle(ParticleBase *p)
-{
-    m_particles.push_back(p);
-}
-
-//----------------------------------------------------------------------------
-void PlayGround::markObjectToRemove(ObjectBase *o, bool autoDelete)
-{
-    for (ObjectIter iter = m_objects.begin(); iter != m_objects.end(); ++iter)
-    {
-        if (iter->object == o)
-        {
-            iter->toRemove = true;
-            iter->autoDelete = autoDelete;
-            break;
-        }
-    }
-}
-
-//----------------------------------------------------------------------------
-const Platform *PlayGround::getPlatform(unsigned number) const
-{
-    FindPlatformByNumberConstVisitor v(number);
-    v.visit(m_objects);
-    return v.getPlatform();
-}
-
-//----------------------------------------------------------------------------
-unsigned PlayGround::countCratesToRescue() const
-{
-    CountCratesToRescueConstVisitor v;
-    v.visit(m_objects);
-    return v.getCratesToRescue();
-}
-
 //----------------------------------------------------------------------------
 Crate *PlayGround::hasCrateFor(const Ship *ship) const
 {
     HasCrateForShipConstVisitor v(ship);
-    v.visit(m_objects);
+    ObjectRepository::getInstance()->acceptObjects(v);
     return const_cast<Crate*>(v.getCrate());
 }
 
 //----------------------------------------------------------------------------
 void PlayGround::resize(Uint16 xTiles, Uint16 yTiles)
-    throw (SDLException)
 {
     m_boundingBox.w = xTiles * 16;
     m_boundingBox.h = yTiles * 16;
@@ -259,7 +188,6 @@ void PlayGround::resize(Uint16 xTiles, Uint16 yTiles)
 
 //----------------------------------------------------------------------------
 void PlayGround::readMapFrom(const char *mission, const char *map)
-    throw (Exception)
 {
     std::string full;
     full.append(Configuration::getInstance()->getDataDir())
@@ -267,12 +195,32 @@ void PlayGround::readMapFrom(const char *mission, const char *map)
 
     File f(full.c_str(), "r");
 
-    if (f.readUint8() != '\x01')
+    Uint8 version = f.readUint8();
+    switch (version)
     {
-        throw Exception("Map file versions other than 0x01 are unsupported");
+    case 0x01:
+        readMapV1(f);
+        break;
+
+    case 0x02:
+        readMapV2(f);
+        break;
+
+    default:
+    {
+        std::ostringstream s;
+        s << "The map file version 0x"
+          << std::setfill('0') << std::setw(2) << std::hex << version
+          << " is not supported";
+
+        throw Exception(s.str());
     }
+    }
+}
 
-
+//----------------------------------------------------------------------------
+void PlayGround::readMapV1(File &f)
+{
     Uint16 xTiles = f.readUint16();
     Uint16 yTiles = f.readUint16();
     resize(xTiles, yTiles);
@@ -297,15 +245,81 @@ void PlayGround::readMapFrom(const char *mission, const char *map)
 
                 Uint8 tile = f.readUint8();
 
-                SDL_CALLS::BlitSurface(
-                    TileCategories::getInstance()[category][tile], 0,
-                    m_mapSurface, &r);
+                if (tile == 0x00)  continue;
 
-                if (category != 0 && category != 0xff && tile != 0)
+                Uint8 alpha = (category == 0xff ? 0x80 : SDL_ALPHA_OPAQUE);
+                const SDL_Surface *tileSurface =
+                    Tiles::getInstance()->getTile(category, tile, alpha, 0x80);
+
+                SDL_CALLS::BlitSurface(tileSurface, NULL, m_mapSurface, &r);
+
+                if (category != 0x00 && category != 0xff)
                 {
                     SDL_CALLS::BlitSurface(
-                        TileCategories::getInstance()[category][tile], 0,
-                        m_collisionSurface, &r);
+                        tileSurface, NULL, m_collisionSurface, &r);
+
+                    m_background.set(x, y, false);
+                }
+            }
+        }
+    }
+    while (--layers > 0);
+
+    SDL_CALLS::BlitSurface(m_mapSurface, NULL, m_shadowSurface, NULL);
+}
+
+//----------------------------------------------------------------------------
+void PlayGround::readMapV2(File &f)
+{
+    Uint16 xTiles = f.readUint16();
+    Uint16 yTiles = f.readUint16();
+    resize(xTiles, yTiles);
+
+    Uint8 layers = f.readUint8();
+    if (layers == 0)
+    {
+        throw Exception("There must be at least one layer");
+    }
+
+    do
+    {
+        Uint8 category = f.readUint8();
+        Uint8 background = f.readUint8();
+        Uint8 alpha = f.readUint8();
+        Uint8 brightness = f.readUint8();
+
+        if (category == 0x00 || category == 0xff)
+        {
+            std::ostringstream s;
+            s << "The category index 0x"
+              << std::setfill('0') << std::setw(2) << std::hex << category
+              << " shall not be used for version 2 maps";
+
+            throw Exception(s.str());
+        }
+
+        for (Uint16 y=0; y<yTiles; y++)
+        {
+            SDL_Rect r;
+            for (Uint16 x=0; x<xTiles; x++)
+            {
+                r.x = 16*x;
+                r.y = 16*y;
+
+                Uint8 tile = f.readUint8();
+
+                if (tile == 0x00)  continue;
+
+                const SDL_Surface *tileSurface =
+                    Tiles::getInstance()->getTile(
+                        category, tile, alpha, brightness);
+
+                SDL_CALLS::BlitSurface(tileSurface, NULL, m_mapSurface, &r);
+
+                if (background == 0x00)
+                {
+                    SDL_CALLS::BlitSurface(
+                        tileSurface, NULL, m_collisionSurface, &r);
 
                     m_background.set(x, y, false);
                 }
@@ -450,17 +464,17 @@ bool PlayGround::collidesWith(const ObjectBase *object) const
 //----------------------------------------------------------------------------
 bool PlayGround::collidesWithParticle(const ParticleBase *particle) const
 {
-    const SDL_Rect &particlePos = particle->getPosition();
+    const SDL_Rect &pos = particle->getPosition();
 
-    return SDL_TOOLS::getPixel(
-        getCollisionSurface(), particlePos.x, particlePos.y) != 0;
+    return SDL_TOOLS::inside(pos.x, pos.y, m_boundingBox) ?
+        SDL_TOOLS::getPixel(getCollisionSurface(), pos.x, pos.y) != 0 : false;
 }
 
 //----------------------------------------------------------------------------
 const Platform *PlayGround::isInLandingZone(const Ship *ship) const
 {
     FindLandingPlatformConstVisitor v(ship);
-    v.visit(m_objects);
+    ObjectRepository::getInstance()->acceptObjects(v);
     return v.getPlatform();
 }
 
@@ -546,70 +560,33 @@ bool PlayGround::do_isBackgroundBetweenY(Uint16 x1, Uint16 y1,
 
 //----------------------------------------------------------------------------
 void PlayGround::update()
-    throw (SDLException)
 {
+    ObjectRepository *r = ObjectRepository::getInstance();
+
+    m_updateRects.clear();
+
     OverdrawObjectsConstVisitor v1(m_mapSurface, m_shadowSurface);
-    v1.visit(m_objects);
-    v1.visit(m_particles);
+    r->acceptAll(v1);
 
     UpdateObjectsVisitor v2;
-    v2.visit(m_objects);
-    v2.visit(m_particles);
+    r->acceptAll(v2);
 
-    CollisionDetectorVisitor v3(m_objects);
-    v3.visit(m_objects);
-    v3.visit(m_particles);
+    CollisionDetectorVisitor v3;
+    r->acceptAll(v3);
 
-    removeObjectsToRemove();
+    r->removeObjectsToRemove();
 
     BlitObjectsConstVisitor v4(m_shadowSurface);
-    v4.visit(m_objects);
-    v4.visit(m_particles);
+    r->acceptAll(v4);
 }
 
 //----------------------------------------------------------------------------
-void PlayGround::updateForPreview() throw (SDLException)
+void PlayGround::updateForPreview()
 {
     BlitObjectsConstVisitor v(m_shadowSurface);
-    v.visit(m_objects);
+    ObjectRepository::getInstance()->acceptObjects(v);
 }
 
-//----------------------------------------------------------------------------
-void PlayGround::removeObjectsToRemove()
-{
-    ObjectIter iter = m_objects.begin();
-    while (iter != m_objects.end())
-    {
-        if (iter->toRemove)
-        {
-            if (iter->autoDelete)
-            {
-                delete iter->object;
-            }
-            iter = m_objects.erase(iter);
-        }
-        else
-        {
-            ++iter;
-        }
-    }
-
-    ParticleIter iter2 = m_particles.begin();
-    while (iter2 != m_particles.end())
-    {
-        if ((*iter2)->isRemove())
-        {
-            delete *iter2;
-            iter2 = m_particles.erase(iter2);
-        }
-        else
-        {
-            ++iter2;
-        }
-    }
-}
-
-
 
 //----------------------------------------------------------------------------
 void OverdrawObjectsConstVisitor::do_visit(const ParticleBase *p)
@@ -697,7 +674,7 @@ void CollisionDetectorVisitor::ParticleCollisionDetectorVisitor::do_onCollision(
 {
     if (ObjectBase::isCollision(m_particle, o))
     {
-        m_particle->setRemove();
+        m_particle->setHidden(true);
         setCollision();
     }
 }
@@ -709,6 +686,7 @@ void CollisionDetectorVisitor::ShipCollisionDetectorVisitor::do_visit(GrenadeBas
     if (ObjectBase::isCollision(m_ship, g))
     {
         GameControlBase::getInstance()->onCollision(m_ship, g);
+        setCollision();
     }
 }
 
@@ -718,6 +696,7 @@ void CollisionDetectorVisitor::ShipCollisionDetectorVisitor::do_visit(Missile *m
     if (ObjectBase::isCollision(m_ship, m))
     {
         GameControlBase::getInstance()->onCollision(m_ship, m);
+        setCollision();
     }
 }
 
@@ -727,6 +706,7 @@ void CollisionDetectorVisitor::ShipCollisionDetectorVisitor::do_visit(MortarBase
     if (ObjectBase::isCollision(m_ship, m))
     {
         GameControlBase::getInstance()->onCollision(m_ship, m);
+        setCollision();
     }
 }
 
@@ -736,6 +716,7 @@ void CollisionDetectorVisitor::ShipCollisionDetectorVisitor::do_visit(Projectile
     if (ObjectBase::isCollision(m_ship, p))
     {
         GameControlBase::getInstance()->onCollision(m_ship, p);
+        setCollision();
     }
 }
 
@@ -745,6 +726,7 @@ void CollisionDetectorVisitor::ShipCollisionDetectorVisitor::do_visit(SAMBattery
     if (ObjectBase::isCollision(m_ship, s))
     {
         GameControlBase::getInstance()->onCollision(m_ship, s);
+        setCollision();
     }
 }
 
@@ -754,6 +736,7 @@ void CollisionDetectorVisitor::ShipCollisionDetectorVisitor::do_visit(Tank *t)
     if (ObjectBase::isCollision(m_ship, t))
     {
         GameControlBase::getInstance()->onCollision(m_ship, t);
+        setCollision();
     }
 }
 
@@ -763,6 +746,7 @@ void CollisionDetectorVisitor::ShipCollisionDetectorVisitor::do_visit(TurretBase
     if (ObjectBase::isCollision(m_ship, t))
     {
         GameControlBase::getInstance()->onCollision(m_ship, t);
+        setCollision();
     }
 }
 
@@ -772,6 +756,7 @@ void CollisionDetectorVisitor::ShipCollisionDetectorVisitor::do_onCollision(Obje
     if (ObjectBase::isCollision(m_ship, object))
     {
         GameControlBase::getInstance()->onCollision(m_ship);
+        setCollision();
     }
 }
 
@@ -782,6 +767,7 @@ void CollisionDetectorVisitor::MissileCollisionDetectorVisitor::do_visit(Grenade
     if (ObjectBase::isCollision(m_missile, g))
     {
         GameControlBase::getInstance()->onCollision(m_missile, g);
+        setCollision();
     }
 }
 
@@ -791,6 +777,7 @@ void CollisionDetectorVisitor::MissileCollisionDetectorVisitor::do_visit(MortarB
     if (ObjectBase::isCollision(m_missile, m))
     {
         GameControlBase::getInstance()->onCollision(m_missile, m);
+        setCollision();
     }
 }
 
@@ -800,6 +787,7 @@ void CollisionDetectorVisitor::MissileCollisionDetectorVisitor::do_visit(Project
     if (ObjectBase::isCollision(m_missile, p))
     {
         GameControlBase::getInstance()->onCollision(m_missile, p);
+        setCollision();
     }
 }
 
@@ -809,6 +797,17 @@ void CollisionDetectorVisitor::MissileCollisionDetectorVisitor::do_visit(SAMBatt
     if (ObjectBase::isCollision(m_missile, s))
     {
         GameControlBase::getInstance()->onCollision(m_missile, s);
+        setCollision();
+    }
+}
+
+//----------------------------------------------------------------------------
+void CollisionDetectorVisitor::MissileCollisionDetectorVisitor::do_visit(SwitchBase *s)
+{
+    if (ObjectBase::isCollision(m_missile, s))
+    {
+        GameControlBase::getInstance()->onCollision(m_missile, s);
+        setCollision();
     }
 }
 
@@ -818,6 +817,7 @@ void CollisionDetectorVisitor::MissileCollisionDetectorVisitor::do_visit(TurretB
     if (ObjectBase::isCollision(m_missile, t))
     {
         GameControlBase::getInstance()->onCollision(m_missile, t);
+        setCollision();
     }
 }
 
@@ -827,6 +827,7 @@ void CollisionDetectorVisitor::MissileCollisionDetectorVisitor::do_visit(Tank *t
     if (ObjectBase::isCollision(m_missile, t))
     {
         GameControlBase::getInstance()->onCollision(m_missile, t);
+        setCollision();
     }
 }
 
@@ -836,6 +837,7 @@ void CollisionDetectorVisitor::MissileCollisionDetectorVisitor::do_onCollision(O
     if (ObjectBase::isCollision(m_missile, o))
     {
         GameControlBase::getInstance()->onCollision(m_missile);
+        setCollision();
     }
 }
 
@@ -846,6 +848,17 @@ void CollisionDetectorVisitor::GrenadeCollisionDetectorVisitor::do_visit(MortarB
     if (ObjectBase::isCollision(m_grenade, m))
     {
         GameControlBase::getInstance()->onCollision(m_grenade, m);
+        setCollision();
+    }
+}
+
+//----------------------------------------------------------------------------
+void CollisionDetectorVisitor::GrenadeCollisionDetectorVisitor::do_visit(SwitchBase *s)
+{
+    if (ObjectBase::isCollision(m_grenade, s))
+    {
+        GameControlBase::getInstance()->onCollision(m_grenade, s);
+        setCollision();
     }
 }
 
@@ -855,6 +868,7 @@ void CollisionDetectorVisitor::GrenadeCollisionDetectorVisitor::do_visit(Tank *t
     if (ObjectBase::isCollision(m_grenade, t))
     {
         GameControlBase::getInstance()->onCollision(m_grenade, t);
+        setCollision();
     }
 }
 
@@ -864,6 +878,7 @@ void CollisionDetectorVisitor::GrenadeCollisionDetectorVisitor::do_visit(TurretB
     if (ObjectBase::isCollision(m_grenade, t))
     {
         GameControlBase::getInstance()->onCollision(m_grenade, t);
+        setCollision();
     }
 }
 
@@ -873,6 +888,7 @@ void CollisionDetectorVisitor::GrenadeCollisionDetectorVisitor::do_onCollision(O
     if (ObjectBase::isCollision(m_grenade, o))
     {
         GameControlBase::getInstance()->onCollision(m_grenade);
+        setCollision();
     }
 }
 
@@ -883,6 +899,17 @@ void CollisionDetectorVisitor::ProjectileCollisionDetectorVisitor::do_visit(Mort
     if (ObjectBase::isCollision(m_projectile, m))
     {
         GameControlBase::getInstance()->onCollision(m_projectile, m);
+        setCollision();
+    }
+}
+
+//----------------------------------------------------------------------------
+void CollisionDetectorVisitor::ProjectileCollisionDetectorVisitor::do_visit(SwitchBase *s)
+{
+    if (ObjectBase::isCollision(m_projectile, s))
+    {
+        GameControlBase::getInstance()->onCollision(m_projectile, s);
+        setCollision();
     }
 }
 
@@ -892,6 +919,7 @@ void CollisionDetectorVisitor::ProjectileCollisionDetectorVisitor::do_visit(Tank
     if (ObjectBase::isCollision(m_projectile, t))
     {
         GameControlBase::getInstance()->onCollision(m_projectile, t);
+        setCollision();
     }
 }
 
@@ -901,6 +929,7 @@ void CollisionDetectorVisitor::ProjectileCollisionDetectorVisitor::do_visit(Turr
     if (ObjectBase::isCollision(m_projectile, t))
     {
         GameControlBase::getInstance()->onCollision(m_projectile, t);
+        setCollision();
     }
 }
 
@@ -910,6 +939,7 @@ void CollisionDetectorVisitor::ProjectileCollisionDetectorVisitor::do_onCollisio
     if (ObjectBase::isCollision(m_projectile, o))
     {
         GameControlBase::getInstance()->onCollision(m_projectile);
+        setCollision();
     }
 }
 
@@ -921,12 +951,12 @@ void CollisionDetectorVisitor::do_visit(ParticleBase *p)
 
     if (playGround->collidesWithParticle(p))
     {
-        p->setRemove();
+        p->setHidden(true);
     }
     else
     {
         ParticleCollisionDetectorVisitor v(p);
-        v.visit(m_objects);
+        ObjectRepository::getInstance()->acceptObjects(v);
     }
 }
 
@@ -942,7 +972,7 @@ void CollisionDetectorVisitor::do_visit(Ship *s)
     else
     {
         ShipCollisionDetectorVisitor v(s);
-        v.visit(m_objects);
+        ObjectRepository::getInstance()->acceptObjects(v);
     }
 }
 
@@ -958,7 +988,7 @@ void CollisionDetectorVisitor::do_visit(Missile *m)
     else
     {
         MissileCollisionDetectorVisitor v(m);
-        v.visit(m_objects);
+        ObjectRepository::getInstance()->acceptObjects(v);
     }
 }
 
@@ -974,7 +1004,7 @@ void CollisionDetectorVisitor::do_visit(GrenadeBase *g)
     else
     {
         GrenadeCollisionDetectorVisitor v(g);
-        v.visit(m_objects);
+        ObjectRepository::getInstance()->acceptObjects(v);
     }
 }
 
@@ -990,7 +1020,7 @@ void CollisionDetectorVisitor::do_visit(ProjectileBase *p)
     else
     {
         ProjectileCollisionDetectorVisitor v(p);
-        v.visit(m_objects);
+        ObjectRepository::getInstance()->acceptObjects(v);
     }
 }
 
