@@ -41,7 +41,7 @@ class LevelSelectorMenu : public Menu
 
   private:
 
-    void initMissionSelect() throw(GuiException);
+    void initMissionSelect();
 
     ReplaceableSurfaceWidget *m_previewWidget;
 
