@@ -11,6 +11,17 @@
 class DecorationBase : public ObjectBase
 {
   public:
+    //------------------------------------------------------------------------
+    class InitializationData
+    {
+      public:
+        InitializationData(const XMLNode *node);
+        virtual ~InitializationData();
+
+        /// The object id of the decoration.
+        unsigned id;
+    };
+
     //------------------------------------------------------------------------
     virtual ~DecorationBase();
 
@@ -21,15 +32,16 @@ class DecorationBase : public ObjectBase
      *
      * @return A pointer to the created decoration.
      */
-    static DecorationBase *create(const XMLNode *decorationNode)
-        throw (Exception);
+    static DecorationBase *create(const XMLNode *decorationNode);
 
-    /// @return true, if the decoration has clear vision to the player's ship.
+    /**
+     * @return true, if the decoration has clear vision to the player's ship.
+     */
     bool canReachPlayerShip() const;
 
   protected:
     //------------------------------------------------------------------------
-    DecorationBase();
+    DecorationBase(const InitializationData &init);
 };
 
 #endif //DECORATIONBASE_H
