@@ -6,23 +6,9 @@
 #include "GameInterface.h"
 #include "SoundInterface.h"
 
-#include "MovingObjectBase.h"
-#include   "ControllableObjectBase.h"
-#include     "Grinder.h"
-#include     "Missile.h"
-#include     "Ship.h"
-#include   "Particle.h"
-#include   "Projectile.h"
-#include   "Grenade.h"
-
-#include "ObjectVisitors.h"
-#include "Crates.h"
-#include "Mortars.h"
-#include "ParticleFountains.h"
-#include "SAMBattery.h"
-#include "Turrets.h"
-#include "Tank.h"
+#include "ObjectRepository.h"
 
+#include "AllObjects.h"
 #include "ObjectVisitors.h"
 
 
@@ -31,8 +17,8 @@
 //============================================================================
 
 //----------------------------------------------------------------------------
-MovingObjectBase::MovingObjectBase(const ObjectBase *creator)
-        : ObjectBase(creator)
+MovingObjectBase::MovingObjectBase(const unsigned creatorId)
+        : ObjectBase(0, creatorId)
 {
 }
 
@@ -48,8 +34,8 @@ MovingObjectBase::~MovingObjectBase()
 //============================================================================
 
 //----------------------------------------------------------------------------
-ControllableObjectBase::ControllableObjectBase(const ObjectBase *creator)
-        : MovingObjectBase(creator)
+ControllableObjectBase::ControllableObjectBase(const unsigned creatorId)
+        : MovingObjectBase(creatorId)
 {
     m_angle = SURFACES_ROTATION_STEPS / 2;
     m_fineAngle = 180.0;
@@ -115,17 +101,6 @@ void ControllableObjectBase::setThrust(bool enable)
     }
 }
 
-//----------------------------------------------------------------------------
-void ControllableObjectBase::createExplosionParticles()
-{
-    unsigned numParticles = getPosition().w * getPosition().h / 4;
-    for (unsigned i=0; i<numParticles; i++)
-    {
-        GameInterface::getInstance()->addParticleToPlayGround(
-            new ExplosionParticle(this));
-    }
-}
-
 //----------------------------------------------------------------------------
 void ControllableObjectBase::updateRotation()
 {
@@ -195,7 +170,7 @@ void ControllableObjectBase::updateVelocityAndPosition()
         fineAccel.setExponential(
             1.0 * getThrustForce() / getMass(), 1.0 * getAngle());
 
-        gameInterface->addParticleToPlayGround(new ThrustParticle(this));
+        ObjectRepository::getInstance()->addObject(new ThrustParticle(this));
     }
 
     // Apply the playground's gravity force.
@@ -357,7 +332,7 @@ void Missile::NormalWarheadStrategy::updateWarhead(Missile *missile)
 }
 
 //----------------------------------------------------------------------------
-void Missile::NormalWarheadStrategy::onCreateExplosionParticles(Missile *missile)
+void Missile::NormalWarheadStrategy::onCreateExplosionParticles(const Missile *missile) const
 {
 }
 
@@ -369,7 +344,7 @@ void Missile::ConeBurstWarheadStrategy::updateWarhead(Missile *missile)
 }
 
 //----------------------------------------------------------------------------
-void Missile::ConeBurstWarheadStrategy::onCreateExplosionParticles(Missile *missile)
+void Missile::ConeBurstWarheadStrategy::onCreateExplosionParticles(const Missile *missile) const
 {
     // If the missile crashed or was shot down,
     // no projectiles shall be created.
@@ -379,7 +354,7 @@ void Missile::ConeBurstWarheadStrategy::onCreateExplosionParticles(Missile *miss
         int angle = (int)missile->getAngle() - 45;
         for (int i=0; i<=90; i+=15)
         {
-            GameInterface::getInstance()->addObjectToPlayGround(
+            ObjectRepository::getInstance()->addObject(
                 new MissileProjectile(missile, angle+i));
         }
     }
@@ -393,7 +368,7 @@ void Missile::StarBurstWarheadStrategy::updateWarhead(Missile *missile)
 }
 
 //----------------------------------------------------------------------------
-void Missile::StarBurstWarheadStrategy::onCreateExplosionParticles(Missile *missile)
+void Missile::StarBurstWarheadStrategy::onCreateExplosionParticles(const Missile *missile) const
 {
     // If the missile crashed or was shot down,
     // no projectiles shall be created.
@@ -402,7 +377,7 @@ void Missile::StarBurstWarheadStrategy::onCreateExplosionParticles(Missile *miss
     {
         for (int angle=0; angle<360; angle+=45)
         {
-            GameInterface::getInstance()->addObjectToPlayGround(
+            ObjectRepository::getInstance()->addObject(
                 new MissileProjectile(missile, angle));
         }
     }
@@ -413,8 +388,8 @@ void Missile::StarBurstWarheadStrategy::onCreateExplosionParticles(Missile *miss
 //----------------------------------------------------------------------------
 Missile::Missile(EnumHeadingStrategy heading,
                  EnumWarheadStrategy warhead,
-                 const ObjectBase *creator)
-        : ControllableObjectBase(creator)
+                 const unsigned creatorId)
+        : ControllableObjectBase(creatorId)
 {
     m_mass = 20;
     m_thrust = true;
@@ -434,14 +409,6 @@ Missile::~Missile()
     m_warheadStrategy = NULL;
 }
 
-//----------------------------------------------------------------------------
-void Missile::createExplosionParticles()
-{
-    ControllableObjectBase::createExplosionParticles();
-
-    m_warheadStrategy->onCreateExplosionParticles(this);
-}
-
 //----------------------------------------------------------------------------
 void Missile::updateHeading()
 {
@@ -497,7 +464,7 @@ const SDL_Surface *Missile::getRotationSurface(unsigned frame) const
 }
 
 //----------------------------------------------------------------------------
-DECLARE_OBJECT_VISITOR_API_BODY(Missile);
+DECLARE_OBJECT_VISITOR_API_BODY(Missile)
 
 
 
@@ -598,7 +565,7 @@ void Ship::Flying::update(Ship *s)
 
 
 //----------------------------------------------------------------------------
-Ship::Ship(ShipSurfaces::Ship shipType) : ControllableObjectBase(NULL)
+Ship::Ship(ShipSurfaces::Ship shipType) : ControllableObjectBase(0)
 {
     m_shipType = shipType;
     m_mass = 100;
@@ -636,7 +603,10 @@ bool Ship::decFuel(unsigned decrement)
 bool Ship::hasCapacity(const Crate *crate) const
 {
     unsigned sum = 0;
-    for (CrateCIter iter = m_crates.begin(); iter != m_crates.end(); ++iter)
+    for (CrateCIter
+             iter = m_crates.begin(),
+             end = m_crates.end();
+         iter != end; ++iter)
     {
         sum += (*iter)->getMass();
     }
@@ -664,16 +634,6 @@ Crate *Ship::popCrate()
     return crate;
 }
 
-//----------------------------------------------------------------------------
-void Ship::removeAllCrates()
-{
-    while (!m_crates.empty())
-    {
-        delete popCrate();
-    }
-}
-
-
 
 //----------------------------------------------------------------------------
 bool Ship::isInLandingZone(const SDL_Rect &r) const
@@ -744,14 +704,13 @@ void Ship::updateFire()
     {
         m_fire = false;
         SoundInterface::getInstance()->onShipShoot();
-        GameInterface::getInstance()->addObjectToPlayGround(
-            new ShipProjectile(this));
+        ObjectRepository::getInstance()->addObject(new ShipProjectile(this));
         GameInterface::getInstance()->onShipFiring(this);
     }
 }
 
 //----------------------------------------------------------------------------
-DECLARE_OBJECT_VISITOR_API_BODY(Ship);
+DECLARE_OBJECT_VISITOR_API_BODY(Ship)
 
 
 
@@ -761,10 +720,9 @@ DECLARE_OBJECT_VISITOR_API_BODY(Ship);
 
 //----------------------------------------------------------------------------
 ParticleBase::ParticleBase(const ObjectBase *creator)
-        : MovingObjectBase(creator)
+        : MovingObjectBase(creator->getId())
 {
     m_lifeCount = 0;
-    m_remove = false;
     m_gravity = false;
 
     m_gradient = false;
@@ -826,13 +784,13 @@ void ParticleBase::update()
 
         if (m_gradientFineIndex >= 100*PARTICLES_GRADIENT_SIZE)
         {
-            setRemove();
+            setHidden(true);
         }
     }
 
     if (--m_lifeCount == 0)
     {
-        setRemove();
+        setHidden(true);
     }
 }
 
@@ -844,12 +802,12 @@ void ParticleBase::updateSurface()
 }
 
 //----------------------------------------------------------------------------
-DECLARE_OBJECT_VISITOR_API_BODY(ParticleBase);
+DECLARE_OBJECT_VISITOR_API_BODY(ParticleBase)
 
 
 //----------------------------------------------------------------------------
-ThrustParticle::ThrustParticle(const ControllableObjectBase *object)
-        : ParticleBase(object)
+ThrustParticle::ThrustParticle(const ObjectBase *creator)
+        : ParticleBase(creator)
 {
     setLifeCount(20 + myRand(5));
     setGradient(200);
@@ -866,7 +824,8 @@ ThrustParticle::~ThrustParticle()
 void ThrustParticle::init()
 {
     const ControllableObjectBase *object =
-        static_cast<const ControllableObjectBase*>(getCreator());
+        static_cast<const ControllableObjectBase*>(
+            ObjectRepository::getInstance()->getObject(getCreatorId()));
 
     // Initialize the position.
 
@@ -895,11 +854,11 @@ void ThrustParticle::init()
 
 
 //----------------------------------------------------------------------------
-ExplosionParticle::ExplosionParticle(const ObjectBase *object)
-        : ParticleBase(object)
+ExplosionParticle::ExplosionParticle(const ObjectBase *creator)
+        : ParticleBase(creator)
 {
     InitializeExplosionParticleConstVisitor v(this);
-    object->accept(&v);
+    creator->accept(v);
 }
 
 //----------------------------------------------------------------------------
@@ -929,7 +888,8 @@ FountainParticle::~FountainParticle()
 void FountainParticle::init()
 {
     const ParticleFountainBase *f =
-        static_cast<const ParticleFountainBase*>(getCreator());
+        static_cast<const ParticleFountainBase*>(
+            ObjectRepository::getInstance()->getObject(getCreatorId()));
 
     // Initialize the position.
 
@@ -961,8 +921,8 @@ void FountainParticle::init()
 //============================================================================
 
 //----------------------------------------------------------------------------
-ProjectileBase::ProjectileBase(const ObjectBase *creator)
-        : MovingObjectBase(creator)
+ProjectileBase::ProjectileBase(const unsigned creatorId)
+        : MovingObjectBase(creatorId)
 {
     setSurface(ProjectileSurfaces::getInstance()->getSurface());
 }
@@ -984,23 +944,12 @@ void ProjectileBase::update()
 }
 
 //----------------------------------------------------------------------------
-void ProjectileBase::createExplosionParticles()
-{
-    unsigned numParticles = 5;
-    for (unsigned i=0; i<numParticles; i++)
-    {
-        GameInterface::getInstance()->addParticleToPlayGround(
-            new ExplosionParticle(this));
-    }
-}
-
-//----------------------------------------------------------------------------
-DECLARE_OBJECT_VISITOR_API_BODY(ProjectileBase);
+DECLARE_OBJECT_VISITOR_API_BODY(ProjectileBase)
 
 
 //----------------------------------------------------------------------------
-ShipProjectile::ShipProjectile(Ship *shooter) 
-        : ProjectileBase(shooter)
+ShipProjectile::ShipProjectile(const Ship *shooter) 
+        : ProjectileBase(shooter->getId())
 {
     // Calculate the ship's rotation point.
     const SDL_Rect &shipPosition = shooter->getPosition();
@@ -1034,15 +983,29 @@ ShipProjectile::~ShipProjectile()
 
 
 //----------------------------------------------------------------------------
-TurretProjectile::TurretProjectile(TurretBase *turret,
+TurretProjectile::TurretProjectile(const TurretBase *turret,
                                    const TurretBarrel *barrel)
-        : ProjectileBase(turret)
+        : ProjectileBase(turret->getId())
 {
     // Set the initial position.
     Sint16 x, y;
-    turret->getTipPosition(x, y);
-    x -= getPosition().w / 2;
-    y -= getPosition().h / 2;
+    switch (turret->getTipPosition(x, y))
+    {
+    case OrientatingDecorationBase::O_TOP:
+        x -= getPosition().w / 2;
+        break;
+    case OrientatingDecorationBase::O_BOTTOM:
+        x -= getPosition().w / 2;
+        y -= getPosition().h;
+        break;
+    case OrientatingDecorationBase::O_LEFT:
+        y -= getPosition().h / 2;
+        break;
+    case OrientatingDecorationBase::O_RIGHT:
+        x -= getPosition().w;
+        y -= getPosition().h / 2;
+        break;
+    }
 
     setPosition(x, y);
 
@@ -1059,8 +1022,8 @@ TurretProjectile::~TurretProjectile()
 
 
 //----------------------------------------------------------------------------
-MissileProjectile::MissileProjectile(Missile *missile, int angle)
-        : ProjectileBase(missile->getCreator())
+MissileProjectile::MissileProjectile(const Missile *missile, int angle)
+        : ProjectileBase(missile->getCreatorId())
 {
     // Set the initial position.
     Sint16 x, y;
@@ -1086,8 +1049,8 @@ MissileProjectile::~MissileProjectile()
 
 
 //----------------------------------------------------------------------------
-TankProjectile::TankProjectile(Tank *tank)
-        : ProjectileBase(tank)
+TankProjectile::TankProjectile(const Tank *tank)
+        : ProjectileBase(tank->getId())
 {
     // Set the initial position.
     Sint16 x, y;
@@ -1112,8 +1075,8 @@ TankProjectile::~TankProjectile()
 
 
 //----------------------------------------------------------------------------
-GrenadeProjectile::GrenadeProjectile(GrenadeBase *grenade, int angle)
-        : ProjectileBase(grenade->getCreator())
+GrenadeProjectile::GrenadeProjectile(const GrenadeBase *grenade, int angle)
+        : ProjectileBase(grenade->getCreatorId())
 {
     // Set the initial position.
     Sint16 x, y;
@@ -1163,12 +1126,12 @@ GrenadeBase::WarheadStrategy *GrenadeBase::WarheadStrategy::getStrategy(
 }
 
 //----------------------------------------------------------------------------
-void GrenadeBase::NoWarheadStrategy::onCreateExplosionParticles(GrenadeBase *missile)
+void GrenadeBase::NoWarheadStrategy::onCreateExplosionParticles(const GrenadeBase *missile) const
 {
 }
 
 //----------------------------------------------------------------------------
-void GrenadeBase::StarBurstWarheadStrategy::onCreateExplosionParticles(GrenadeBase *grenade)
+void GrenadeBase::StarBurstWarheadStrategy::onCreateExplosionParticles(const GrenadeBase *grenade) const
 {
     // If the grenade crashed, no projectiles shall be created.
 
@@ -1176,7 +1139,7 @@ void GrenadeBase::StarBurstWarheadStrategy::onCreateExplosionParticles(GrenadeBa
     {
         for (int angle=0; angle<360; angle+=45)
         {
-            GameInterface::getInstance()->addObjectToPlayGround(
+            ObjectRepository::getInstance()->addObject(
                 new GrenadeProjectile(grenade, angle));
         }
     }
@@ -1185,8 +1148,8 @@ void GrenadeBase::StarBurstWarheadStrategy::onCreateExplosionParticles(GrenadeBa
 
 //----------------------------------------------------------------------------
 GrenadeBase::GrenadeBase(EnumWarheadStrategy warhead,
-                         const ObjectBase *creator)
-        : MovingObjectBase(creator)
+                         const unsigned creatorId)
+        : MovingObjectBase(creatorId)
 {
     m_lifeCount = 0;
     m_explode = false;
@@ -1232,31 +1195,32 @@ void GrenadeBase::update()
 }
 
 //----------------------------------------------------------------------------
-void GrenadeBase::createExplosionParticles()
-{
-    unsigned numParticles = 15;
-    for (unsigned i=0; i<numParticles; i++)
-    {
-        GameInterface::getInstance()->addParticleToPlayGround(
-            new ExplosionParticle(this));
-    }
-
-    m_warheadStrategy->onCreateExplosionParticles(this);
-}
-
-//----------------------------------------------------------------------------
-DECLARE_OBJECT_VISITOR_API_BODY(GrenadeBase);
+DECLARE_OBJECT_VISITOR_API_BODY(GrenadeBase)
 
 
 //----------------------------------------------------------------------------
 MortarGrenade::MortarGrenade(MortarBase *mortar, const MortarBarrel *barrel)
-        : GrenadeBase(barrel->getGrenadeWarheadStrategy(), mortar)
+        : GrenadeBase(barrel->getGrenadeWarheadStrategy(), mortar->getId())
 {
     // Set the initial position.
     Sint16 x, y;
-    mortar->getTipPosition(x, y);
-    x -= getPosition().w / 2;
-    y -= getPosition().h / 2;
+    switch (mortar->getTipPosition(x, y))
+    {
+    case OrientatingDecorationBase::O_TOP:
+        x -= getPosition().w / 2;
+        break;
+    case OrientatingDecorationBase::O_BOTTOM:
+        x -= getPosition().w / 2;
+        y -= getPosition().h;
+        break;
+    case OrientatingDecorationBase::O_LEFT:
+        y -= getPosition().h / 2;
+        break;
+    case OrientatingDecorationBase::O_RIGHT:
+        x -= getPosition().w;
+        y -= getPosition().h / 2;
+        break;
+    }
 
     setPosition(x, y);
 
