@@ -1,31 +1,19 @@
 #include "Configuration.h"
 
-#include "Barriers.h"
-#include "BlackHole.h"
-#include "ControllableObjectBase.h"
-#include   "Missile.h"
-#include   "Ship.h"
-#include "Crates.h"
-#include "Magnets.h"
-#include "Particle.h"
-#include "ParticleFountains.h"
-#include "Platform.h"
-#include "Projectile.h"
-#include "Grenade.h"
-#include "SAMBattery.h"
-#include "Turrets.h"
-#include "Mortars.h"
-#include "Grinder.h"
-#include "Tank.h"
+#include "AllObjects.h"
 
 
 //============================================================================
 // Implementation of SurfaceBase.h
 //============================================================================
 
+//----------------------------------------------------------------------------
+SurfacesBase::~SurfacesBase()
+{
+}
+
 //----------------------------------------------------------------------------
 void SurfacesBase::init()
-    throw (Exception)
 {
     MagnetSurfaces::init();
     PlatformSurfaces::init();
@@ -34,6 +22,7 @@ void SurfacesBase::init()
     BlackHoleSurfaces::init();
     BarrierSurfaces::init();
     SAMBatterySurfaces::init();
+    SwitchSurfaces::init();
 
     CrateSurfaces::init();
     GrinderSurfaces::init();
@@ -59,6 +48,7 @@ void SurfacesBase::destroy()
     GrinderSurfaces::destroy();
     CrateSurfaces::destroy();
 
+    SwitchSurfaces::destroy();
     SAMBatterySurfaces::destroy();
     BarrierSurfaces::destroy();
     BlackHoleSurfaces::destroy();
@@ -88,7 +78,6 @@ SingleSurfacesBase::~SingleSurfacesBase()
 
 //----------------------------------------------------------------------------
 void SingleSurfacesBase::initSurfaceFrom(const char *file)
-    throw (SDLException)
 {
     m_surface = SDL_TOOLS::loadBMPWithColorKey(file);
 }
@@ -116,7 +105,6 @@ OrientatingSurfacesBase::~OrientatingSurfacesBase()
 
 //----------------------------------------------------------------------------
 void OrientatingSurfacesBase::initSurfacesFrom(const char *file)
-    throw (SDLException)
 {
     SDL_Surface *s = SDL_CALLS::LoadBMP(file);
 
@@ -155,7 +143,6 @@ RotatingSurfacesBase::~RotatingSurfacesBase()
 
 //----------------------------------------------------------------------------
 void RotatingSurfacesBase::initSurfacesFrom(const char *file)
-    throw (SDLException)
 {
     SDL_Surface *s = SDL_CALLS::LoadBMP(file);
 
@@ -234,7 +221,6 @@ BarrierSurfaces::~BarrierSurfaces()
 
 //----------------------------------------------------------------------------
 void BarrierSurfaces::init()
-    throw (Exception)
 {
     std::string dir;
     dir.append(Configuration::getInstance()->getDataDir())
@@ -331,7 +317,7 @@ SDL_Surface *BarrierSurfaces::createSurface(HVDecorationBase::Orientation o,
     SDL_Rect r = { 0, 0, 16, 16 };
 
     // The left corner tile can always be blitted directly.
-    SDL_CALLS::BlitSurface(m_left[o], 0, s, &r);
+    SDL_CALLS::BlitSurface(m_left[o], NULL, s, &r);
 
     // Blit the center tiles.
     switch (o)
@@ -340,7 +326,7 @@ SDL_Surface *BarrierSurfaces::createSurface(HVDecorationBase::Orientation o,
         for (r.x = 16; r.x < (Sint16)size*16 - 16; r.x += 16)
         {
             SDL_CALLS::BlitSurface(
-                activated ? m_beam[o] : m_deactivated, 0, s, &r);
+                activated ? m_beam[o] : m_deactivated, NULL, s, &r);
         }
         break;
 
@@ -348,13 +334,13 @@ SDL_Surface *BarrierSurfaces::createSurface(HVDecorationBase::Orientation o,
         for (r.y = 16; r.y < (Sint16)size*16 - 16; r.y += 16)
         {
             SDL_CALLS::BlitSurface(
-                activated ? m_beam[o] : m_deactivated, 0, s, &r);
+                activated ? m_beam[o] : m_deactivated, NULL, s, &r);
         }
         break;
     }
 
     // The right corner tile can be blitted directly too.
-    SDL_CALLS::BlitSurface(m_right[o], 0, s, &r);
+    SDL_CALLS::BlitSurface(m_right[o], NULL, s, &r);
 
     SDL_CALLS::SetColorKey(
         s, SDL_SRCCOLORKEY, SDL_MapRGB(s->format, 0, 0, 0));
@@ -383,7 +369,6 @@ BlackHoleSurfaces::~BlackHoleSurfaces()
 
 //----------------------------------------------------------------------------
 void BlackHoleSurfaces::init()
-    throw (Exception)
 {
     sm_instance = new BlackHoleSurfaces();
 
@@ -421,7 +406,6 @@ MissileSurfaces::~MissileSurfaces()
 
 //----------------------------------------------------------------------------
 void MissileSurfaces::init()
-    throw (Exception)
 {
     sm_instance = new MissileSurfaces();
 
@@ -467,7 +451,6 @@ ShipSurfaces::~ShipSurfaces()
 
 //----------------------------------------------------------------------------
 void ShipSurfaces::init()
-    throw (Exception)
 {
     const char *file[] = {
         "default.bmp", "arrow.bmp",
@@ -568,7 +551,6 @@ CrateSurfaces::~CrateSurfaces()
 
 //----------------------------------------------------------------------------
 void CrateSurfaces::init()
-    throw (Exception)
 {
     static const char *file[] =
     {
@@ -636,7 +618,6 @@ MagnetSurfaces::~MagnetSurfaces()
 
 //----------------------------------------------------------------------------
 void MagnetSurfaces::init()
-    throw (Exception)
 {
     std::string dir;
     dir.append(Configuration::getInstance()->getDataDir())
@@ -739,12 +720,12 @@ SDL_Surface *MagnetSurfaces::createSurface(OrientatingDecorationBase::Orientatio
     {
     case OrientatingDecorationBase::O_TOP:
     case OrientatingDecorationBase::O_RIGHT:
-        SDL_CALLS::BlitSurface(m_right[o][frame], 0, s, &r);
+        SDL_CALLS::BlitSurface(m_right[o][frame], NULL, s, &r);
         break;
 
     case OrientatingDecorationBase::O_BOTTOM:
     case OrientatingDecorationBase::O_LEFT:
-        SDL_CALLS::BlitSurface(m_left[o][frame], 0, s, &r);
+        SDL_CALLS::BlitSurface(m_left[o][frame], NULL, s, &r);
         break;
     }
 
@@ -755,7 +736,7 @@ SDL_Surface *MagnetSurfaces::createSurface(OrientatingDecorationBase::Orientatio
     case OrientatingDecorationBase::O_BOTTOM:
         for (r.x = 16; r.x < (Sint16)size - 16; r.x += 16)
         {
-            SDL_CALLS::BlitSurface(m_center[o][frame], 0, s, &r);
+            SDL_CALLS::BlitSurface(m_center[o][frame], NULL, s, &r);
         }
         break;
 
@@ -763,7 +744,7 @@ SDL_Surface *MagnetSurfaces::createSurface(OrientatingDecorationBase::Orientatio
     case OrientatingDecorationBase::O_RIGHT:
         for (r.y = 16; r.y < (Sint16)size - 16; r.y += 16)
         {
-            SDL_CALLS::BlitSurface(m_center[o][frame], 0, s, &r);
+            SDL_CALLS::BlitSurface(m_center[o][frame], NULL, s, &r);
         }
         break;
     }
@@ -773,12 +754,12 @@ SDL_Surface *MagnetSurfaces::createSurface(OrientatingDecorationBase::Orientatio
     {
     case OrientatingDecorationBase::O_TOP:
     case OrientatingDecorationBase::O_RIGHT:
-        SDL_CALLS::BlitSurface(m_left[o][frame], 0, s, &r);
+        SDL_CALLS::BlitSurface(m_left[o][frame], NULL, s, &r);
         break;
 
     case OrientatingDecorationBase::O_BOTTOM:
     case OrientatingDecorationBase::O_LEFT:
-        SDL_CALLS::BlitSurface(m_right[o][frame], 0, s, &r);
+        SDL_CALLS::BlitSurface(m_right[o][frame], NULL, s, &r);
         break;
     }
 
@@ -816,7 +797,6 @@ ParticleSurfaces::~ParticleSurfaces()
 
 //----------------------------------------------------------------------------
 void ParticleSurfaces::init()
-    throw (Exception)
 {
     sm_instance = new ParticleSurfaces();
 
@@ -880,7 +860,6 @@ ParticleFountainSurfaces::~ParticleFountainSurfaces()
 
 //----------------------------------------------------------------------------
 void ParticleFountainSurfaces::init()
-    throw (Exception)
 {
     sm_instance = new ParticleFountainSurfaces();
 
@@ -909,11 +888,9 @@ PlatformSurfaces *PlatformSurfaces::sm_instance = NULL;
 //----------------------------------------------------------------------------
 PlatformSurfaces::PlatformSurfaces()
 {
-    memset(m_leftNumber, 0, sizeof(m_leftNumber));
-    memset(m_leftBeacon, 0, sizeof(m_leftBeacon));
+    memset(m_left, 0, sizeof(m_left));
     m_center = NULL;
-    memset(m_rightNumber, 0, sizeof(m_rightNumber));
-    memset(m_rightBeacon, 0, sizeof(m_rightBeacon));
+    memset(m_right, 0, sizeof(m_right));
 }
 
 //----------------------------------------------------------------------------
@@ -927,27 +904,20 @@ PlatformSurfaces::~PlatformSurfaces()
     }
     m_surfaceMap.clear();
 
-    for (int c = BC_RED; c < BC_TOTAL_NUMBER; c++)
+    for (int c=0; c < ET_TOTAL_NUMBER; c++)
     {
         for (int i=0; i<2; i++)
         {
-            ZAP_SURFACE(m_leftBeacon[c][i]);
-            ZAP_SURFACE(m_rightBeacon[c][i]);
+            ZAP_SURFACE(m_left[c][i]);
+            ZAP_SURFACE(m_right[c][i]);
         }
     }
 
-    for (int i=0; i<10; i++)
-    {
-        ZAP_SURFACE(m_leftNumber[i]);
-        ZAP_SURFACE(m_rightNumber[i]);
-    }
-
     ZAP_SURFACE(m_center);
 }
 
 //----------------------------------------------------------------------------
 void PlatformSurfaces::init()
-    throw (Exception)
 {
     std::string dir;
     dir.append(Configuration::getInstance()->getDataDir())
@@ -962,31 +932,31 @@ void PlatformSurfaces::init()
         "red", "yellow", "green", "cyan", "blue", "magenta"
     };
 
-    for (int c = BC_RED; c < BC_TOTAL_NUMBER; c++)
+    for (int c = ET_NO_EDGE; c <= ET_NUMBER_9; c++)
+    {
+        snprintf(file, 1024, "%s/leftno%d.bmp", dir.c_str(), c);
+        sm_instance->m_left[c][0] = SDL_CALLS::LoadBMP(file);
+        sm_instance->m_left[c][1] = SDL_CALLS::LoadBMP(file);
+
+        snprintf(file, 1024, "%s/rightno%d.bmp", dir.c_str(), c);
+        sm_instance->m_right[c][0] = SDL_CALLS::LoadBMP(file);
+        sm_instance->m_right[c][1] = SDL_CALLS::LoadBMP(file);
+    }
+
+    for (int c = ET_RED; c < ET_TOTAL_NUMBER; c++)
     {
         for (int i=0; i<=1; i++)
         {
             snprintf(file, 1024, "%s/%sleft%d.bmp",
-                     dir.c_str(), color[c], i+1);
-            sm_instance->m_leftBeacon[c][i] = SDL_CALLS::LoadBMP(file);
+                     dir.c_str(), color[c-ET_RED], i+1);
+            sm_instance->m_left[c][i] = SDL_CALLS::LoadBMP(file);
 
             snprintf(file, 1024, "%s/%sright%d.bmp",
-                     dir.c_str(), color[c], i+1);
-            sm_instance->m_rightBeacon[c][i] =
-                SDL_TOOLS::loadBMPWithColorKey(file);
+                     dir.c_str(), color[c-ET_RED], i+1);
+            sm_instance->m_right[c][i] = SDL_CALLS::LoadBMP(file);
         }
     }
 
-    // Load the number edge tiles.
-    for (int i=0; i<10; i++)
-    {
-        snprintf(file, 1024, "%s/leftno%d.bmp", dir.c_str(), i);
-        sm_instance->m_leftNumber[i] = SDL_CALLS::LoadBMP(file);
-        
-        snprintf(file, 1024, "%s/rightno%d.bmp", dir.c_str(), i);
-        sm_instance->m_rightNumber[i] = SDL_CALLS::LoadBMP(file);
-    }
-
     snprintf(file, 1024, "%s/center.bmp", dir.c_str());
     sm_instance->m_center = SDL_CALLS::LoadBMP(file);
 }
@@ -999,17 +969,15 @@ void PlatformSurfaces::destroy()
 
 //----------------------------------------------------------------------------
 const SDL_Surface *PlatformSurfaces::getSurface(
-    BeaconColor leftBeacon, BeaconColor rightBeacon,
-    unsigned number, unsigned frame, unsigned size)
+    EdgeType left, EdgeType right, unsigned frame, unsigned size)
 {
-    assert(leftBeacon >= BC_NO_BEACON && leftBeacon < BC_TOTAL_NUMBER);
-    assert(rightBeacon >= BC_NO_BEACON && rightBeacon < BC_TOTAL_NUMBER);
-    assert(number < 10);
+    assert(left >= ET_NO_EDGE && left < ET_TOTAL_NUMBER);
+    assert(right >= ET_NO_EDGE && right < ET_TOTAL_NUMBER);
     assert(frame < 2);
 
     SDL_Surface *s = NULL;
 
-    SurfaceKey key(leftBeacon, rightBeacon, number, frame, size);
+    SurfaceKey key(left, right, frame, size);
     SurfaceCIter iter = m_surfaceMap.find(key);
     if (iter != m_surfaceMap.end())
     {
@@ -1017,7 +985,7 @@ const SDL_Surface *PlatformSurfaces::getSurface(
     }
     else
     {
-        s = createSurface(leftBeacon, rightBeacon, number, frame, size);
+        s = createSurface(left, right, frame, size);
         m_surfaceMap[key] = s;
     }
 
@@ -1026,37 +994,22 @@ const SDL_Surface *PlatformSurfaces::getSurface(
 
 //----------------------------------------------------------------------------
 SDL_Surface *PlatformSurfaces::createSurface(
-    BeaconColor leftBeacon, BeaconColor rightBeacon,
-    unsigned number, unsigned frame, unsigned size) const
+    EdgeType left, EdgeType right, unsigned frame, unsigned size) const
 {
     SDL_Surface *s = SDL_CALLS::CreateRGBSurface(SDL_SWSURFACE, size, 16, 32);
     SDL_Rect r = { 0, 0, 16, 16 };
 
     // The left corner tile can always be blitted directly.
-    if (leftBeacon == BC_NO_BEACON)
-    {
-        SDL_CALLS::BlitSurface(m_leftNumber[number], 0, s, &r);
-    }
-    else
-    {
-        SDL_CALLS::BlitSurface(m_leftBeacon[leftBeacon][frame], 0, s, &r);
-    }
+    SDL_CALLS::BlitSurface(m_left[left][frame], NULL, s, &r);
 
     // Blit the center tiles.
     for (r.x = 16; r.x < (Sint16)size - 16; r.x += 16)
     {
-        SDL_CALLS::BlitSurface(m_center, 0, s, &r);
+        SDL_CALLS::BlitSurface(m_center, NULL, s, &r);
     }
 
     // The right corner tile can be blitted directly too.
-    if (rightBeacon == BC_NO_BEACON)
-    {
-        SDL_CALLS::BlitSurface(m_rightNumber[number], 0, s, &r);
-    }
-    else
-    {
-        SDL_CALLS::BlitSurface(m_rightBeacon[rightBeacon][frame], 0, s, &r);
-    }
+    SDL_CALLS::BlitSurface(m_right[right][frame], NULL, s, &r);
 
     SDL_CALLS::SetColorKey(
         s, SDL_SRCCOLORKEY, SDL_MapRGB(s->format, 0, 0, 0));
@@ -1085,7 +1038,6 @@ ProjectileSurfaces::~ProjectileSurfaces()
 
 //----------------------------------------------------------------------------
 void ProjectileSurfaces::init()
-    throw (Exception)
 {
     sm_instance = new ProjectileSurfaces();
 
@@ -1123,7 +1075,6 @@ GrenadeSurfaces::~GrenadeSurfaces()
 
 //----------------------------------------------------------------------------
 void GrenadeSurfaces::init()
-    throw (Exception)
 {
     sm_instance = new GrenadeSurfaces();
 
@@ -1161,7 +1112,6 @@ SAMBatterySurfaces::~SAMBatterySurfaces()
 
 //----------------------------------------------------------------------------
 void SAMBatterySurfaces::init()
-    throw (Exception)
 {
     sm_instance = new SAMBatterySurfaces();
 
@@ -1180,6 +1130,44 @@ void SAMBatterySurfaces::destroy()
 
 
 
+//============================================================================
+// Implementation of SwitchSurfaces.h
+//============================================================================
+
+//----------------------------------------------------------------------------
+SwitchSurfaces *SwitchSurfaces::sm_instance = NULL;
+
+//----------------------------------------------------------------------------
+SwitchSurfaces::SwitchSurfaces()
+{
+}
+
+//----------------------------------------------------------------------------
+SwitchSurfaces::~SwitchSurfaces()
+{
+}
+
+//----------------------------------------------------------------------------
+void SwitchSurfaces::init()
+{
+    sm_instance = new SwitchSurfaces();
+
+    std::string path;
+    path.append(Configuration::getInstance()->getDataDir())
+        .append("/gfx/decoration/").append("switch.bmp");
+
+    sm_instance->initSurfacesFrom(path.c_str());
+}
+
+//----------------------------------------------------------------------------
+void SwitchSurfaces::destroy()
+{
+    ZAP_POINTER(sm_instance);
+}
+
+
+
+
 //============================================================================
 // Implementation of TurretSurfaces.h
 //============================================================================
@@ -1207,7 +1195,6 @@ TurretSurfaces::~TurretSurfaces()
 
 //----------------------------------------------------------------------------
 void TurretSurfaces::init()
-    throw (Exception)
 {
     static const char *file[] =
     {
@@ -1284,7 +1271,6 @@ GrinderSurfaces::~GrinderSurfaces()
 
 //----------------------------------------------------------------------------
 void GrinderSurfaces::init()
-    throw (Exception)
 {
     sm_instance = new GrinderSurfaces();
 
@@ -1322,7 +1308,6 @@ TankSurfaces::~TankSurfaces()
 
 //----------------------------------------------------------------------------
 void TankSurfaces::init()
-    throw (Exception)
 {
     sm_instance = new TankSurfaces();
 
