@@ -6,14 +6,10 @@
 #include "SoundInterface.h"
 
 #include "Tiles.h"
-#include "DecorationBase.h"
+#include "AllObjects.h"
 #include "GameControlBase.h"
 #include "PlayGround.h"
 
-#include "Magnets.h"
-#include "Missile.h"
-#include "Ship.h"
-
 #include "GameBridge.h"
 
 
@@ -36,7 +32,7 @@ public:
         SDLFrameRate::setFrameRate(72);
 
         // @todo: Find a way to test the objects without using libMoaggGame.
-        TileCategories::init();
+        Tiles::init();
         SurfacesBase::init();
 
         GameInterface::setInstance(GameBridge::getInstance());
@@ -49,7 +45,7 @@ public:
         GameControlBase::destroy();
 
         SurfacesBase::destroy();
-        TileCategories::destroy();
+        Tiles::destroy();
     }
 
 
@@ -138,7 +134,6 @@ protected:
         }
 
         // The first update activates the magnets.
-        playGround->clearUpdateRects();
         playGround->update();
 
         // Now check the gravity field again.
@@ -165,7 +160,6 @@ protected:
         // After fps-1 update()'s, the magnets will be turned off again.
         for (unsigned i=1; i<SDLFrameRate::getFrameRate(); i++)
         {
-            playGround->clearUpdateRects();
             playGround->update();
         }
 
@@ -188,6 +182,233 @@ protected:
 
 
 
+//----------------------------------------------------------------------------
+class TurretTest : public CppUnit::TestFixture
+{
+    CPPUNIT_TEST_SUITE(TurretTest);
+
+    CPPUNIT_TEST(testNoCollisionWithCreatedProjectiles);
+
+    CPPUNIT_TEST_SUITE_END();
+
+public:
+
+    //------------------------------------------------------------------------
+    void setUp()
+    {
+        Configuration::getInstance()->searchDataDir();
+
+        SDLFrameRate::setFrameRate(72);
+
+        // @todo: Find a way to test the objects without using libMoaggGame.
+        Tiles::init();
+        SurfacesBase::init();
+
+        GameInterface::setInstance(GameBridge::getInstance());
+    }
+
+    //------------------------------------------------------------------------
+    void tearDown() 
+    {
+        SurfacesBase::destroy();
+        Tiles::destroy();
+    }
+
+
+protected:
+
+    //------------------------------------------------------------------------
+    void testNoCollisionWithCreatedProjectiles()
+    {
+        XMLNode turretNode("turret");
+        turretNode.addProperty(new XMLProperty("orientation", ""));
+        turretNode.addProperty(new XMLProperty("x", "0"));
+        turretNode.addProperty(new XMLProperty("y", "0"));
+        turretNode.addProperty(new XMLProperty("type", "tube"));
+        turretNode.addProperty(new XMLProperty("delay", "10"));
+
+        XMLNode barrelNode("barrel");
+        barrelNode.addProperty(new XMLProperty("type", "fixed"));
+        barrelNode.addProperty(new XMLProperty("angle", "0"));
+
+        do
+        {
+            turretNode.getProperty("orientation")->setValue("top");
+            TurretBase::InitializationData turretInit(&turretNode);
+            TurretBarrel::InitializationData barrelInit(&barrelNode);
+
+            TopTurret turret(turretInit);
+            TurretBarrel barrel(barrelInit);
+
+            TurretProjectile projectile(&turret, &barrel);
+
+            CPPUNIT_ASSERT(!ObjectBase::isCollision(&turret, &projectile));
+        }
+        while (0);
+
+        do
+        {
+            turretNode.getProperty("orientation")->setValue("bottom");
+            TurretBase::InitializationData turretInit(&turretNode);
+            TurretBarrel::InitializationData barrelInit(&barrelNode);
+
+            BottomTurret turret(turretInit);
+            TurretBarrel barrel(barrelInit);
+
+            TurretProjectile projectile(&turret, &barrel);
+
+            CPPUNIT_ASSERT(!ObjectBase::isCollision(&turret, &projectile));
+        }
+        while (0);
+
+        do
+        {
+            turretNode.getProperty("orientation")->setValue("left");
+            TurretBase::InitializationData turretInit(&turretNode);
+            TurretBarrel::InitializationData barrelInit(&barrelNode);
+
+            LeftTurret turret(turretInit);
+            TurretBarrel barrel(barrelInit);
+
+            TurretProjectile projectile(&turret, &barrel);
+
+            CPPUNIT_ASSERT(!ObjectBase::isCollision(&turret, &projectile));
+        }
+        while (0);
+
+        do
+        {
+            turretNode.getProperty("orientation")->setValue("right");
+            TurretBase::InitializationData turretInit(&turretNode);
+            TurretBarrel::InitializationData barrelInit(&barrelNode);
+
+            RightTurret turret(turretInit);
+            TurretBarrel barrel(barrelInit);
+
+            TurretProjectile projectile(&turret, &barrel);
+
+            CPPUNIT_ASSERT(!ObjectBase::isCollision(&turret, &projectile));
+        }
+        while (0);
+    }
+};
+
+
+
+//----------------------------------------------------------------------------
+class MortarTest : public CppUnit::TestFixture
+{
+    CPPUNIT_TEST_SUITE(MortarTest);
+
+    CPPUNIT_TEST(testNoCollisionWithCreatedGrenades);
+
+    CPPUNIT_TEST_SUITE_END();
+
+public:
+
+    //------------------------------------------------------------------------
+    void setUp()
+    {
+        Configuration::getInstance()->searchDataDir();
+
+        SDLFrameRate::setFrameRate(72);
+
+        // @todo: Find a way to test the objects without using libMoaggGame.
+        Tiles::init();
+        SurfacesBase::init();
+
+        GameInterface::setInstance(GameBridge::getInstance());
+    }
+
+    //------------------------------------------------------------------------
+    void tearDown() 
+    {
+        SurfacesBase::destroy();
+        Tiles::destroy();
+    }
+
+
+protected:
+
+    //------------------------------------------------------------------------
+    void testNoCollisionWithCreatedGrenades()
+    {
+        XMLNode mortarNode("mortar");
+        mortarNode.addProperty(new XMLProperty("orientation", ""));
+        mortarNode.addProperty(new XMLProperty("x", "0"));
+        mortarNode.addProperty(new XMLProperty("y", "0"));
+        mortarNode.addProperty(new XMLProperty("delay", "10"));
+
+        XMLNode barrelNode("barrel");
+        barrelNode.addProperty(new XMLProperty("type", "fixed"));
+        barrelNode.addProperty(new XMLProperty("angle", "0"));
+        barrelNode.addProperty(new XMLProperty("warhead", "none"));
+        barrelNode.addProperty(new XMLProperty("delay", "10"));
+
+        do
+        {
+            mortarNode.getProperty("orientation")->setValue("top");
+            MortarBase::InitializationData mortarInit(&mortarNode);
+            MortarBarrel::InitializationData barrelInit(&barrelNode);
+
+            TopMortar mortar(mortarInit);
+            MortarBarrel barrel(barrelInit);
+
+            MortarGrenade grenade(&mortar, &barrel);
+
+            CPPUNIT_ASSERT(!ObjectBase::isCollision(&mortar, &grenade));
+        }
+        while (0);
+
+        do
+        {
+            mortarNode.getProperty("orientation")->setValue("bottom");
+            MortarBase::InitializationData mortarInit(&mortarNode);
+            MortarBarrel::InitializationData barrelInit(&barrelNode);
+
+            BottomMortar mortar(mortarInit);
+            MortarBarrel barrel(barrelInit);
+
+            MortarGrenade grenade(&mortar, &barrel);
+
+            CPPUNIT_ASSERT(!ObjectBase::isCollision(&mortar, &grenade));
+        }
+        while (0);
+
+        do
+        {
+            mortarNode.getProperty("orientation")->setValue("left");
+            MortarBase::InitializationData mortarInit(&mortarNode);
+            MortarBarrel::InitializationData barrelInit(&barrelNode);
+
+            LeftMortar mortar(mortarInit);
+            MortarBarrel barrel(barrelInit);
+
+            MortarGrenade grenade(&mortar, &barrel);
+
+            CPPUNIT_ASSERT(!ObjectBase::isCollision(&mortar, &grenade));
+        }
+        while (0);
+
+        do
+        {
+            mortarNode.getProperty("orientation")->setValue("right");
+            MortarBase::InitializationData mortarInit(&mortarNode);
+            MortarBarrel::InitializationData barrelInit(&barrelNode);
+
+            RightMortar mortar(mortarInit);
+            MortarBarrel barrel(barrelInit);
+
+            MortarGrenade grenade(&mortar, &barrel);
+
+            CPPUNIT_ASSERT(!ObjectBase::isCollision(&mortar, &grenade));
+        }
+        while (0);
+    }
+};
+
+
+
 //----------------------------------------------------------------------------
 class MissileTest : public CppUnit::TestFixture
 {
@@ -518,5 +739,7 @@ protected:
 
 //----------------------------------------------------------------------------
 CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(MagnetTest, "Magnet");
+CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(TurretTest, "Turret");
+CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(MortarTest, "Mortar");
 CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(MissileTest, "Missile");
 CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(ShipTest, "Ship");
