@@ -2,6 +2,9 @@
 #include <cppunit/extensions/HelperMacros.h>
 
 #include "Tools.h"
+#include "SyscallException.h"
+#include "StringTokenizer.h"
+#include "StringTools.h"
 #include "MathTools.h"
 #include "SystemCalls.h"
 #include "File.h"
@@ -10,7 +13,276 @@
 #include "XMLParser.h"
 #include "SDLTools.h"
 
-#include <iostream>
+
+//----------------------------------------------------------------------------
+class StringTokenizerTest : public CppUnit::TestFixture
+{
+    CPPUNIT_TEST_SUITE(StringTokenizerTest);
+
+    CPPUNIT_TEST(testNormalTokens);
+    CPPUNIT_TEST(testEmptyTokens);
+    CPPUNIT_TEST(testEmptyTokenString);
+
+    CPPUNIT_TEST_SUITE_END();
+
+public:
+
+    //------------------------------------------------------------------------
+    void setUp()
+    {
+    }
+
+    //------------------------------------------------------------------------
+    void tearDown() 
+    {
+    }
+
+protected:
+
+    //------------------------------------------------------------------------
+    void testNormalTokens()
+    {
+        StringTokenizer st("1;2:3;4", ";:");
+
+        std::string t;
+        CPPUNIT_ASSERT(st.hasMoreTokens());
+        CPPUNIT_ASSERT_EQUAL((size_t)4, st.countTokens());
+        CPPUNIT_ASSERT_EQUAL(std::string("1"), st.nextToken(t));
+        CPPUNIT_ASSERT(st.hasMoreTokens());
+        CPPUNIT_ASSERT_EQUAL((size_t)3, st.countTokens());
+        CPPUNIT_ASSERT_EQUAL(std::string("2"), st.nextToken(t));
+        CPPUNIT_ASSERT(st.hasMoreTokens());
+        CPPUNIT_ASSERT_EQUAL((size_t)2, st.countTokens());
+        CPPUNIT_ASSERT_EQUAL(std::string("3"), st.nextToken(t));
+        CPPUNIT_ASSERT(st.hasMoreTokens());
+        CPPUNIT_ASSERT_EQUAL((size_t)1, st.countTokens());
+        CPPUNIT_ASSERT_EQUAL(std::string("4"), st.nextToken(t));
+        CPPUNIT_ASSERT(!st.hasMoreTokens());
+        CPPUNIT_ASSERT_EQUAL((size_t)0, st.countTokens());
+        CPPUNIT_ASSERT_EQUAL(std::string(""), st.nextToken(t));
+        CPPUNIT_ASSERT(!st.hasMoreTokens());
+        CPPUNIT_ASSERT_EQUAL((size_t)0, st.countTokens());
+    }
+
+    //------------------------------------------------------------------------
+    void testEmptyTokens()
+    {
+        StringTokenizer st(";:;", ";:");
+
+        std::string empty;
+        std::string t;
+        CPPUNIT_ASSERT(st.hasMoreTokens());
+        CPPUNIT_ASSERT_EQUAL((size_t)4, st.countTokens());
+        CPPUNIT_ASSERT_EQUAL(empty, st.nextToken(t));
+        CPPUNIT_ASSERT(st.hasMoreTokens());
+        CPPUNIT_ASSERT_EQUAL((size_t)3, st.countTokens());
+        CPPUNIT_ASSERT_EQUAL(empty, st.nextToken(t));
+        CPPUNIT_ASSERT(st.hasMoreTokens());
+        CPPUNIT_ASSERT_EQUAL((size_t)2, st.countTokens());
+        CPPUNIT_ASSERT_EQUAL(empty, st.nextToken(t));
+        CPPUNIT_ASSERT(st.hasMoreTokens());
+        CPPUNIT_ASSERT_EQUAL((size_t)1, st.countTokens());
+        CPPUNIT_ASSERT_EQUAL(empty, st.nextToken(t));
+        CPPUNIT_ASSERT(!st.hasMoreTokens());
+        CPPUNIT_ASSERT_EQUAL((size_t)0, st.countTokens());
+    }
+
+    //------------------------------------------------------------------------
+    void testEmptyTokenString()
+    {
+        StringTokenizer st("", ";:");
+
+        std::string empty;
+        std::string t;
+        CPPUNIT_ASSERT(!st.hasMoreTokens());
+        CPPUNIT_ASSERT_EQUAL((size_t)0, st.countTokens());
+        CPPUNIT_ASSERT_EQUAL(empty, st.nextToken(t));
+    }
+};
+
+
+
+//----------------------------------------------------------------------------
+class StringToolsTest : public CppUnit::TestFixture
+{
+    CPPUNIT_TEST_SUITE(StringToolsTest);
+
+    CPPUNIT_TEST(testToBool);
+    CPPUNIT_TEST(testToInt);
+    CPPUNIT_TEST(testToUnsigned);
+
+    CPPUNIT_TEST(testHasUnsignedInTokenString);
+    CPPUNIT_TEST(testPopUnsignedFromTokenString);
+    CPPUNIT_TEST(testPopFirstUnsignedFromTokenString);
+    CPPUNIT_TEST(testConvertNumberRanges);
+
+    CPPUNIT_TEST_SUITE_END();
+
+public:
+
+    //------------------------------------------------------------------------
+    void setUp()
+    {
+    }
+    
+    //------------------------------------------------------------------------
+    void tearDown()
+    {
+    }
+
+protected:
+
+    //------------------------------------------------------------------------
+    void testToBool()
+    {
+        CPPUNIT_ASSERT(STRING_TOOLS::toBool("1"));
+        CPPUNIT_ASSERT(STRING_TOOLS::toBool("true"));
+        CPPUNIT_ASSERT(STRING_TOOLS::toBool("True"));
+        CPPUNIT_ASSERT(STRING_TOOLS::toBool("TRUE"));
+        CPPUNIT_ASSERT(STRING_TOOLS::toBool("on"));
+        CPPUNIT_ASSERT(STRING_TOOLS::toBool("On"));
+        CPPUNIT_ASSERT(STRING_TOOLS::toBool("ON"));
+        CPPUNIT_ASSERT(STRING_TOOLS::toBool("yes"));
+        CPPUNIT_ASSERT(STRING_TOOLS::toBool("Yes"));
+        CPPUNIT_ASSERT(STRING_TOOLS::toBool("YES"));
+
+        CPPUNIT_ASSERT(!STRING_TOOLS::toBool("0"));
+        CPPUNIT_ASSERT(!STRING_TOOLS::toBool("false"));
+        CPPUNIT_ASSERT(!STRING_TOOLS::toBool("False"));
+        CPPUNIT_ASSERT(!STRING_TOOLS::toBool("FALSE"));
+        CPPUNIT_ASSERT(!STRING_TOOLS::toBool("off"));
+        CPPUNIT_ASSERT(!STRING_TOOLS::toBool("Off"));
+        CPPUNIT_ASSERT(!STRING_TOOLS::toBool("OFF"));
+        CPPUNIT_ASSERT(!STRING_TOOLS::toBool("no"));
+        CPPUNIT_ASSERT(!STRING_TOOLS::toBool("No"));
+        CPPUNIT_ASSERT(!STRING_TOOLS::toBool("NO"));
+
+        try {
+            (void)STRING_TOOLS::toBool("");
+            CPPUNIT_FAIL("toBool(\"\") didn't throw an exception");
+        } catch (Exception &e) {}
+
+        try {
+            (void)STRING_TOOLS::toBool("2");
+            CPPUNIT_FAIL("toBool(\"2\") didn't throw an exception");
+        } catch (Exception &e) {}
+
+        try {
+            (void)STRING_TOOLS::toBool("x");
+            CPPUNIT_FAIL("toBool(\"x\") didn't throw an exception");
+        } catch (Exception &e) {}
+    }
+
+    //------------------------------------------------------------------------
+    void testToInt()
+    {
+        CPPUNIT_ASSERT_EQUAL((int)0, STRING_TOOLS::toInt("0"));
+        CPPUNIT_ASSERT_EQUAL((int)-1, STRING_TOOLS::toInt("-1"));
+        CPPUNIT_ASSERT_EQUAL((int)1, STRING_TOOLS::toInt("1"));
+        
+        try {
+            (void)STRING_TOOLS::toInt("");
+            CPPUNIT_FAIL("toInt(\"\") didn't throw an exception");
+        } catch (Exception &e) {}
+
+        try {
+            (void)STRING_TOOLS::toInt("x");
+            CPPUNIT_FAIL("toInt(\"x\") didn't throw an exception");
+        } catch (Exception &e) {}
+    }
+
+    //------------------------------------------------------------------------
+    void testToUnsigned()
+    {
+        CPPUNIT_ASSERT_EQUAL((unsigned)0, STRING_TOOLS::toUnsigned("0"));
+        CPPUNIT_ASSERT_EQUAL((unsigned)1, STRING_TOOLS::toUnsigned("1"));
+        
+        try {
+            (void)STRING_TOOLS::toInt("");
+            CPPUNIT_FAIL("toUnsigned(\"\") didn't throw an exception");
+        } catch (Exception &e) {}
+
+        // @todo: Negative numbers shall throw an exception!
+        // try {
+        //     (void)STRING_TOOLS::toUnsigned("-1");
+        //     CPPUNIT_FAIL("toUnsigned(\"-1\") didn't throw an exception");
+        // } catch (Exception &e) {}
+
+        try {
+            (void)STRING_TOOLS::toUnsigned("x");
+            CPPUNIT_FAIL("toUnsigned(\"x\") didn't throw an exception");
+        } catch (Exception &e) {}
+    }
+
+    //------------------------------------------------------------------------
+    void testHasUnsignedInTokenString()
+    {
+        std::string s("1,2,4,5");
+
+        CPPUNIT_ASSERT(STRING_TOOLS::hasUnsignedInTokenString(s, 1));
+        CPPUNIT_ASSERT(STRING_TOOLS::hasUnsignedInTokenString(s, 2));
+        CPPUNIT_ASSERT(!STRING_TOOLS::hasUnsignedInTokenString(s, 3));
+        CPPUNIT_ASSERT(STRING_TOOLS::hasUnsignedInTokenString(s, 4));
+        CPPUNIT_ASSERT(STRING_TOOLS::hasUnsignedInTokenString(s, 5));
+
+        std::string empty;
+
+        CPPUNIT_ASSERT(!STRING_TOOLS::hasUnsignedInTokenString(empty, 1));
+    }
+
+    //------------------------------------------------------------------------
+    void testPopUnsignedFromTokenString()
+    {
+        std::string s("1,2,4,5");
+
+        CPPUNIT_ASSERT(STRING_TOOLS::popUnsignedFromTokenString(s, 1));
+        CPPUNIT_ASSERT_EQUAL(std::string("2,4,5"), s);
+        CPPUNIT_ASSERT(STRING_TOOLS::popUnsignedFromTokenString(s, 4));
+        CPPUNIT_ASSERT_EQUAL(std::string("2,5"), s);
+        CPPUNIT_ASSERT(!STRING_TOOLS::popUnsignedFromTokenString(s, 3));
+        CPPUNIT_ASSERT_EQUAL(std::string("2,5"), s);
+        CPPUNIT_ASSERT(STRING_TOOLS::popUnsignedFromTokenString(s, 2));
+        CPPUNIT_ASSERT_EQUAL(std::string("5"), s);
+        CPPUNIT_ASSERT(STRING_TOOLS::popUnsignedFromTokenString(s, 5));
+        CPPUNIT_ASSERT_EQUAL(std::string(""), s);
+
+        CPPUNIT_ASSERT(!STRING_TOOLS::popUnsignedFromTokenString(s, 1));
+        CPPUNIT_ASSERT_EQUAL(std::string(""), s);
+    }
+
+    //------------------------------------------------------------------------
+    void testPopFirstUnsignedFromTokenString()
+    {
+        std::string s("1,2,4,5");
+
+        CPPUNIT_ASSERT(STRING_TOOLS::popFirstUnsignedFromTokenString(s, 1));
+        CPPUNIT_ASSERT_EQUAL(std::string("2,4,5"), s);
+        CPPUNIT_ASSERT(!STRING_TOOLS::popFirstUnsignedFromTokenString(s, 4));
+        CPPUNIT_ASSERT_EQUAL(std::string("2,4,5"), s);
+        CPPUNIT_ASSERT(STRING_TOOLS::popFirstUnsignedFromTokenString(s, 2));
+        CPPUNIT_ASSERT_EQUAL(std::string("4,5"), s);
+        CPPUNIT_ASSERT(!STRING_TOOLS::popFirstUnsignedFromTokenString(s, 3));
+        CPPUNIT_ASSERT_EQUAL(std::string("4,5"), s);
+        CPPUNIT_ASSERT(STRING_TOOLS::popFirstUnsignedFromTokenString(s, 4));
+        CPPUNIT_ASSERT_EQUAL(std::string("5"), s);
+        CPPUNIT_ASSERT(STRING_TOOLS::popFirstUnsignedFromTokenString(s, 5));
+        CPPUNIT_ASSERT_EQUAL(std::string(""), s);
+
+        CPPUNIT_ASSERT(!STRING_TOOLS::popFirstUnsignedFromTokenString(s, 1));
+        CPPUNIT_ASSERT_EQUAL(std::string(""), s);
+    }
+
+    //------------------------------------------------------------------------
+    void testConvertNumberRanges()
+    {
+        std::string s("1-3,4,6-7,8,10-12");
+        CPPUNIT_ASSERT_EQUAL(
+            std::string("1,2,3,4,6,7,8,10,11,12"),
+            STRING_TOOLS::convertNumberRanges(s));
+    }
+};
+
+
 
 //----------------------------------------------------------------------------
 class MathToolsTest : public CppUnit::TestFixture
@@ -57,9 +329,10 @@ class FileTest : public CppUnit::TestFixture
 {
     CPPUNIT_TEST_SUITE(FileTest);
 
-    CPPUNIT_TEST(testFileReadWrite);
-    CPPUNIT_TEST(testFileCopy);
-    CPPUNIT_TEST(testFileStat);
+    CPPUNIT_TEST(testReadWrite);
+    CPPUNIT_TEST(testReadWriteUint);
+    CPPUNIT_TEST(testCopy);
+    CPPUNIT_TEST(testStat);
 
     CPPUNIT_TEST_SUITE_END();
 
@@ -79,7 +352,7 @@ public:
 protected:
 
     //------------------------------------------------------------------------
-    void testFileReadWrite()
+    void testReadWrite()
     {
         const char *name = "test.dat";
         const char data[] = "1234567890";
@@ -113,7 +386,43 @@ protected:
     }
 
     //------------------------------------------------------------------------
-    void testFileCopy()
+    void testReadWriteUint()
+    {
+        const char *name = "test.dat";
+        const char data[] = "\x42\x00\xc8\x42";
+        char buffer[sizeof(data)];
+
+        do
+        {
+            File f(name, "w");
+            f.writeUint8(0x42);
+            f.writeUint16(0x00c8);
+            f.writeUint8(0x42);
+        }
+        while (0);
+
+        do
+        {
+            File f(name, "r");
+            f.read(buffer, 4);
+            CPPUNIT_ASSERT_EQUAL(0, memcmp(buffer, data, 4));
+        }
+        while (0);
+
+        do
+        {
+            File f(name, "r");
+            CPPUNIT_ASSERT_EQUAL((uint8_t)0x42, f.readUint8());
+            CPPUNIT_ASSERT_EQUAL((uint16_t)0x00c8, f.readUint16());
+            CPPUNIT_ASSERT_EQUAL((uint8_t)0x42, f.readUint8());
+        }
+        while (0);
+
+        File::unlink(name);
+    }
+
+    //------------------------------------------------------------------------
+    void testCopy()
     {
         const char *src = "test1.dat";
         const char *dst = "test2.dat";
@@ -138,7 +447,7 @@ protected:
     }
 
     //------------------------------------------------------------------------
-    void testFileStat()
+    void testStat()
     {
         const char *name1 = "test1.dat";
         const char *name2 = "test2.dat";
@@ -169,9 +478,9 @@ class DirectoryTest : public CppUnit::TestFixture
 {
     CPPUNIT_TEST_SUITE(DirectoryTest);
 
-    CPPUNIT_TEST(testDirectoryMkdirRmdir);
-    CPPUNIT_TEST(testDirectoryMkdirRmdirRecursive);
-    CPPUNIT_TEST(testDirectoryCopy);
+    CPPUNIT_TEST(testMkdirRmdir);
+    CPPUNIT_TEST(testMkdirRmdirRecursive);
+    CPPUNIT_TEST(testCopy);
 
     CPPUNIT_TEST_SUITE_END();
 
@@ -191,7 +500,7 @@ public:
 protected:
 
     //------------------------------------------------------------------------
-    void testDirectoryMkdirRmdir()
+    void testMkdirRmdir()
     {
         const char *name = "test";
 
@@ -207,7 +516,7 @@ protected:
     }
 
     //------------------------------------------------------------------------
-    void testDirectoryMkdirRmdirRecursive()
+    void testMkdirRmdirRecursive()
     {
         setUpTestDirectory();
 
@@ -221,7 +530,7 @@ protected:
     }
 
     //------------------------------------------------------------------------
-    void testDirectoryCopy()
+    void testCopy()
     {
         setUpTestDirectory();
         Directory::copy("test", "test_copy");
@@ -896,6 +1205,8 @@ protected:
 
 
 //----------------------------------------------------------------------------
+CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(StringTokenizerTest, "StringTokenizer");
+CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(StringToolsTest, "StringTools");
 CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(MathToolsTest, "MathTools");
 CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(FileTest, "File");
 CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(DirectoryTest, "Directory");
